# Generated by Django 5.2.11 on 2026-02-10 16:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('geo_lib', '0005_alter_geoelement_filterable_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='geofeature',
            options={'ordering': ('indicator', 'name'), 'verbose_name': 'Données de la carte', 'verbose_name_plural': 'Données de la carte'},
        ),
    ]



# Generated by Django 5.2.11 on 2026-02-10 16:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('indicators_lib', '0007_remove_subtheme_description_alter_filter_color_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='indicator',
            name='min_mesh',
            field=models.TextField(choices=[('fr', 'Fr'), ('reg', 'Reg'), ('dep', 'Dep'), ('epci', 'Epci'), ('com', 'Com'), ('aom', 'Aom'), ('ept', 'Ept')], default='com'),
        ),
    ]



from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import Group

from territories_dashboard_lib.commons.admin import AdminInfoMixin
from territories_dashboard_lib.website_lib.forms import (
    GlossaryItemAdminForm,
    LandingPageAdminForm,
    MainConfAdminForm,
    StaticPageAdminForm,
)
from territories_dashboard_lib.website_lib.models import (
    GlossaryItem,
    LandingPage,
    MainConf,
    NoticeBanner,
    StaticPage,
)
from territories_dashboard_lib.website_lib.utils import get_website_links


@admin.register(MainConf)
class MainConfAdmin(AdminInfoMixin, admin.ModelAdmin):
    admin_info = "Paramétrage de la configuration générale du site."
    form = MainConfAdminForm

    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj, **kwargs)
        links = ", ".join(get_website_links())
        form.base_fields["header_navigation"].help_text = (
            form.base_fields["header_navigation"].help_text
            + f"<br/><br/>---<br/><br/>Liens disponibles : {links}"
        )
        form.base_fields["footer_navigation"].help_text = (
            form.base_fields["footer_navigation"].help_text
            + f"<br/><br/>---<br/><br/>Liens disponibles : {links}"
        )
        return form


@admin.register(GlossaryItem)
class GlossaryItemAdmin(AdminInfoMixin, admin.ModelAdmin):
    admin_info = "Paramétrage de la page Lexique du site."
    form = GlossaryItemAdminForm


@admin.register(LandingPage)
class LandingPageAdminForm(AdminInfoMixin, admin.ModelAdmin):
    admin_info = "Paramétrage de la page d'accueil du site."
    form = LandingPageAdminForm


@admin.register(StaticPage)
class StaticPageAdminForm(AdminInfoMixin, admin.ModelAdmin):
    admin_info = "Paramétrage des pages statiques du site."
    form = StaticPageAdminForm


@admin.register(NoticeBanner)
class NoticeBannerAdmin(AdminInfoMixin, admin.ModelAdmin):
    admin_info = (
        "Paramétrage des bandeaux d'information à afficher sur les pages du site."
    )
    list_display = ("title", "created_at")


admin.site.unregister(Group)


class CustomUserAdmin(UserAdmin):
    def get_favorite_indicators(self, obj):
        names = obj.favorite_indicators.values_list("title", flat=True)
        return "\n ".join(names) if names else "-"

    get_favorite_indicators.short_description = "Indicateurs favoris"

    readonly_fields = UserAdmin.readonly_fields + ("get_favorite_indicators",)
    fieldsets = UserAdmin.fieldsets + (
        ("Indicateurs favoris", {"fields": ("get_favorite_indicators",)}),
    )


admin.site.unregister(get_user_model())
admin.site.register(get_user_model(), CustomUserAdmin)



# Generated by Django 5.2.10 on 2026-01-19 13:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracking_lib', '0006_alter_event_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='name',
            field=models.TextField(choices=[('download', 'Download'), ('vue-resume-indicateur', 'View Indicator Card'), ('attention-sur-resume-indicateur', 'Attention On Indicator Card')]),
        ),
    ]



from .models import Dashboard


def serialize_dashboard(dashboard: Dashboard):
    return {
        "id": dashboard.id,
        "superset_id": dashboard.superset_id,
        "label": dashboard.label,
        "short_name": dashboard.short_name,
    }



from django.contrib.auth import get_user_model
from django.db import models
from martor.models import MartorField

from territories_dashboard_lib.commons.models import CommonModel
from territories_dashboard_lib.indicators_lib.enums import (
    AggregationFunctions,
    MeshLevel,
)
from territories_dashboard_lib.indicators_lib.refresh_filters import refresh_filters


class Theme(CommonModel):
    ordering = models.IntegerField(
        default=0,
        verbose_name="Ordre dans la sidebar",
        help_text="L'ordre démarre à 0.",
    )
    name = models.CharField(
        max_length=64,
        unique=True,
        verbose_name="Nom (~id, URL)",
        help_text="Pour l'URL, ne mettre que des lettres minuscules sans accents et des tirets.",
    )
    title = models.CharField(max_length=128, verbose_name="Titre (affiché)")
    objectif_theme = models.TextField(blank=True, verbose_name="Objectif")
    action_theme = models.TextField(blank=True, verbose_name="Actions")

    def is_displayed_on_app(self):
        return self.sub_themes.all().exists()

    is_displayed_on_app.__name__ = "Visible ?"
    is_displayed_on_app.boolean = True

    def subthemes_count(self):
        return self.sub_themes.count()

    subthemes_count.__name__ = "Nb sous-thèmes"

    def indicators_count(self):
        return sum(
            [sub_theme.indicators_count() for sub_theme in self.sub_themes.all()]
        )

    indicators_count.__name__ = "Total indicateurs"

    def __str__(self):
        return self.title

    class Meta:
        ordering = ("ordering",)
        verbose_name = "1 - Thème"
        indexes = [
            models.Index(fields=["name"]),
        ]


class SubTheme(CommonModel):
    name = models.CharField(
        max_length=64,
        unique=True,
        verbose_name="Nom (~id, URL)",
        help_text="Pour l'URL, ne mettre que des lettres minuscules sans accents et des tirets.",
    )
    theme = models.ForeignKey(
        Theme, on_delete=models.CASCADE, related_name="sub_themes", verbose_name="Thème"
    )
    index_in_theme = models.IntegerField(
        default=0, verbose_name="Ordre thème", help_text="L'ordre démarre à 0."
    )
    title = models.CharField(max_length=128, verbose_name="Titre (affiché)")

    def __str__(self):
        return f"{self.theme.title} > {self.title}"

    def indicators_count(self):
        return self.indicators.count()

    def is_displayed_on_app(self):
        return self.indicators.all().exists()

    indicators_count.__name__ = "Nb indicateurs"
    is_displayed_on_app.__name__ = "Visible ?"
    is_displayed_on_app.boolean = True

    class Meta:
        ordering = ("theme", "index_in_theme")
        verbose_name = "2 - Sous-thème"
        indexes = [
            models.Index(fields=["theme", "index_in_theme"]),
            models.Index(fields=["name"]),
        ]


class Indicator(CommonModel):
    is_active = models.BooleanField(default=True, verbose_name="Actif ?")
    hidden = models.BooleanField(
        default=False,
        verbose_name="Caché",
        help_text="Coché pour cacher un indicateur actif, il pourra être utilisé en tant que second indicateur sur la carte mais il n'apparaitra pas dans la liste des indicateurs",
    )
    sub_theme = models.ForeignKey(
        SubTheme,
        on_delete=models.CASCADE,
        related_name="indicators",
        verbose_name="Sous-thème",
    )
    index_in_theme = models.IntegerField(
        default=0, verbose_name="Ordre sous-thème", help_text="L'ordre démarre à 0."
    )
    name = models.CharField(
        max_length=32,
        unique=True,
        verbose_name="Nom (~id, URL)",
        help_text="Pour l'URL, ne mettre que des lettres minuscules sans accents et des tirets.",
    )
    title = models.CharField(max_length=128, verbose_name="Titre (affiché)")
    short_title = models.CharField(
        max_length=128,
        null=True,
        blank=True,
        verbose_name="Titre abrégé",
        help_text="Utilisé pour les popups et la légende de la carte.",
    )
    # Indicator's DB attributes
    db_table_prefix = models.CharField(
        max_length=128,
        verbose_name="Préfixe dans la DB",
        help_text="Attention, le nom doit être exactement le même que celui en base.",
    )
    min_mesh = models.TextField(choices=MeshLevel.choices, default=MeshLevel.com)
    is_composite = models.BooleanField(
        default=False, verbose_name="Indicateur composite"
    )
    show_alternative = models.BooleanField(
        default=True,
        verbose_name="Afficher la valeur alternative",
        help_text="Pour certains indicateurs, il n'y a pas de valeur alternative à afficher.",
    )
    aggregation_constant = models.DecimalField(
        default=1,
        decimal_places=5,
        max_digits=10,
        verbose_name="Constante d'agrégation",
    )
    aggregation_function = models.CharField(
        default=AggregationFunctions.REPEATED_COMPONENT_2,
        max_length=8,
        choices=AggregationFunctions.choices,
        verbose_name="Fonction d'agrégation",
    )
    unite = models.CharField(max_length=32, verbose_name="Unité (affichée)")
    unite_nom_accessible = models.CharField(
        max_length=64,
        default="",
        blank=True,
        help_text="Nom accessible de l'unité qui sera lu par le lecteur d'écran.",
    )
    unite_alternative = models.CharField(
        default=None,
        null=True,
        blank=True,
        max_length=32,
        verbose_name="Unité alternative (affichée)",
    )
    unite_alternative_nom_accessible = models.CharField(
        max_length=64,
        default="",
        blank=True,
        help_text="Nom accessible de l'unité alternative qui sera lu par le lecteur d'écran.",
    )
    secondary_indicator = models.ForeignKey(
        "indicators_lib.Indicator",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        verbose_name="Second indicateur à afficher sur la carte",
        help_text="Optionnel.",
    )
    flows_db_table_prefix = models.CharField(
        null=True,
        blank=True,
        max_length=128,
        verbose_name="Table des flux (prefix)",
        help_text="Uniquement pour les indicateurs de flux. Nom de la table des flux.",
    )
    flows_dimension = models.CharField(
        null=True,
        blank=True,
        max_length=32,
        verbose_name="Dimension des flux",
        help_text="Uniquement pour les indicateurs de flux. Nom de la colonne de dimension dans la table des flux.",
    )
    # Descriptive attributes
    show_evolution = models.BooleanField(
        default=True, verbose_name="Activer l'historique"
    )
    source = models.TextField(default="", blank=True)
    description = models.TextField(default="", blank=True)
    methodo = MartorField(
        default="", blank=True, verbose_name="Méthodologie (markdown)"
    )
    methodo_file = models.BinaryField(null=True, blank=True)
    favorited_by = models.ManyToManyField(
        get_user_model(),
        through="UserFavoriteIndicator",
        related_name="favorite_indicators",
    )

    def __str__(self):
        return self.title

    def get_theme_title(self):
        return self.sub_theme.theme.title

    get_theme_title.short_description = "Thème"

    def table_name(indicator, mesh: MeshLevel, *, flows: bool = False):
        table_prefix = (
            indicator.flows_db_table_prefix if flows else indicator.db_table_prefix
        )
        return f"{table_prefix}_{mesh}"

    class Meta:
        ordering = ("sub_theme", "index_in_theme")
        verbose_name = "3 - Indicateur"
        indexes = [
            models.Index(fields=["sub_theme"]),
            models.Index(fields=["name"]),
        ]


class UserFavoriteIndicator(models.Model):
    user = models.ForeignKey(
        get_user_model(),
        on_delete=models.CASCADE,
        related_name="favorite_indicator_links",
    )
    indicator = models.ForeignKey(
        Indicator,
        on_delete=models.CASCADE,
        related_name="favorite_links",
    )
    order = models.PositiveIntegerField(default=0)

    class Meta:
        unique_together = ("user", "indicator")
        ordering = ("user", "order")


class Dimension(CommonModel):
    indicator = models.ForeignKey(
        Indicator, on_delete=models.CASCADE, related_name="dimensions"
    )
    db_name = models.TextField()
    title = models.TextField()
    is_breakdown = models.BooleanField(
        default=False,
        verbose_name="Répartir selon cette dimension",
        help_text="Dans le cas de plusieurs dimensions pour un indicateur, l'une d'entre elles doit être la dimension de réparition pour les graphiques.",
    )

    def save(self, *args, **kwargs):
        is_new = self.pk is None
        result = super().save(*args, **kwargs)
        if is_new:
            try:
                refresh_filters(self)
            except Exception as e:
                print(e)
        return result


class Filter(CommonModel):
    dimension = models.ForeignKey(
        Dimension, related_name="filters", on_delete=models.CASCADE
    )
    db_name = models.CharField(max_length=128, verbose_name="Nom dans la BDD")
    order = models.IntegerField(
        default=0, verbose_name="Ordre", help_text="L'ordre démarre à 0."
    )
    default = models.BooleanField(default=True, verbose_name="Sélectionné par défaut ?")
    color = models.TextField(
        null=True,
        blank=True,
        verbose_name="Couleur",
        help_text="format hexadécimale : #FF11DD. Les données associées à ce filtre apparaitront de cette couleur dans les graphiques.",
    )

    def __str__(self):
        return self.db_name

    class Meta:
        ordering = ("dimension", "order", "db_name")
        verbose_name = "Filtre"
        indexes = [
            models.Index(fields=["db_name"]),
            models.Index(fields=["dimension", "order"]),
        ]
        unique_together = ("dimension", "db_name")



from typing import List, Optional

from pydantic import BaseModel, field_validator

from territories_dashboard_lib.commons.types import TerritoryCode
from territories_dashboard_lib.indicators_lib.enums import MeshLevel
from territories_dashboard_lib.indicators_lib.payloads import SubMeshPayload


def split_comma_separated(v):
    """Split comma-separated strings into a list, handling both strings and lists."""
    if isinstance(v, str):
        return v.split(",")
    if isinstance(v, list):
        result = []
        for item in v:
            if isinstance(item, str) and "," in item:
                result.extend(item.split(","))
            else:
                result.append(item)
        return result
    return v


class GeoFeaturesPayload(SubMeshPayload):
    last: Optional[int] = None
    limit: Optional[int] = 1000
    feature: int


class MainTerritoryParams(BaseModel):
    geo_level: MeshLevel
    geo_id: List[TerritoryCode]

    @field_validator("geo_id", mode="before")
    @classmethod
    def split_geo_id(cls, v):
        return split_comma_separated(v)


class TerritoriesParams(BaseModel):
    mesh: MeshLevel
    territories: List[TerritoryCode]

    @field_validator("territories", mode="before")
    @classmethod
    def split_territories(cls, v):
        return split_comma_separated(v)


class TerritoryFeaturePayload(SubMeshPayload):
    codes: List[TerritoryCode] | None = None

    @field_validator("codes", mode="before")
    @classmethod
    def split_codes(cls, v):
        return split_comma_separated(v)


class SearchTerritoriesParams(BaseModel):
    mesh: MeshLevel
    search: str = ""
    offset: int = 0



# Generated by Django 5.2.9 on 2026-01-06 16:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geo_lib', '0002_geoelement_linked_to_indicator'),
    ]

    operations = [
        migrations.AddField(
            model_name='geofeature',
            name='color_column',
            field=models.TextField(blank=True, help_text="Optionnel. Nom de la colonne qui définit une couleur par point. Les valeurs de la colonne doivent être sous le format #ffffff (hexadecimale). Si renseigné, le champ 'color' ne sera pas utilisé. Seulement pour les points.", null=True, verbose_name='Colonne de la couleur'),
        ),
        migrations.AddField(
            model_name='geofeature',
            name='size_column',
            field=models.TextField(blank=True, help_text="Optionnel. Nom de la colonne qui définit une taille par point. Les valeurs de la colonne doivent être 'sm' (small) ou 'md' (medium). Seulement pour les points.", null=True, verbose_name='Colonne de la taille'),
        ),
    ]



# Generated by Django 5.2.8 on 2025-12-03 13:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('superset_lib', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='filter',
            name='mesh',
            field=models.TextField(blank=True, choices=[('fr', 'Fr'), ('reg', 'Reg'), ('dep', 'Dep'), ('epci', 'Epci'), ('com', 'Com'), ('aom', 'Aom')], help_text="Maille du territoire sur lequel s'effectue le filtre.", null=True),
        ),
    ]



from django.contrib.auth.backends import ModelBackend

from territories_dashboard_lib.auth_lib.models import ProxyUser


class ProxyUserBackend(ModelBackend):
    def get_user(self, user_id):
        try:
            return ProxyUser.objects.get(pk=user_id)
        except ProxyUser.DoesNotExist:
            return None



from django.conf import settings
from django.shortcuts import get_object_or_404

from territories_dashboard_lib.indicators_lib.models import Theme
from territories_dashboard_lib.website_lib.logic import (
    get_favorite_subtheme,
    get_favorite_theme,
)
from territories_dashboard_lib.website_lib.serializers import (
    serialize_indicator,
    serialize_sub_themes,
)


def get_favorite_indicators_ids(request):
    return (
        request.user.favorite_indicators.values_list("id", flat=True)
        if request.user.is_authenticated
        else []
    )


def get_themes(request):
    themes = list(Theme.objects.all().order_by("ordering"))
    if getattr(request.user, "has_favorite_indicators", None):
        favorite_theme = get_favorite_theme()
        themes = [favorite_theme] + themes
    return themes


def get_theme_and_sub_themes(request, theme_name):
    if theme_name == "favoris":
        theme = get_favorite_theme()
        indicators = (
            [
                serialize_indicator(indicator)
                for indicator in (
                    request.user.favorite_indicators.filter(
                        is_active=True, hidden=False
                    ).order_by("favorite_links__order")
                )
            ]
            if request.user.is_authenticated
            else []
        )
        sub_themes = [get_favorite_subtheme(indicators)]
    else:
        theme = get_object_or_404(Theme, name=theme_name)
        sub_themes = serialize_sub_themes(theme)
    return theme, sub_themes


def apply_filters(request, indicator):
    filters = []
    for dimension in indicator.dimensions.all():
        url_filters = request.GET.getlist(dimension.db_name)
        db_filters = [f for f in dimension.filters.all()]
        for f in db_filters:
            f.set_from_url = "!" + f.db_name in url_filters or f.db_name in url_filters
            f.current = (
                f.default
                and "!" + f.db_name not in url_filters
                or f.db_name in url_filters
            )
        filters.append([dimension, db_filters])
    return filters


def get_base_url(request):
    return (
        settings.BASE_URL
        if hasattr(settings, "BASE_URL")
        else request.build_absolute_uri("/")[:-1]
    )



import nested_admin
from django.contrib import admin
from django.db.models import TextField
from django.forms import TextInput
from django.shortcuts import redirect
from django.urls import path, reverse
from django.utils.html import format_html

from territories_dashboard_lib.commons.admin import AdminInfoMixin
from territories_dashboard_lib.indicators_lib.methodo_pdf import reset_methodo_file
from territories_dashboard_lib.indicators_lib.models import (
    Dimension,
    Filter,
    Indicator,
    SubTheme,
    Theme,
)
from territories_dashboard_lib.indicators_lib.refresh_filters import refresh_filters


class FilterInline(nested_admin.NestedTabularInline):  # type: ignore
    model = Filter
    extra = 0
    formfield_overrides = {TextField: {"widget": TextInput(attrs={"size": "32"})}}


class DimensionInline(nested_admin.NestedTabularInline):  # type: ignore
    model = Dimension
    extra = 0
    inlines = [FilterInline]
    formfield_overrides = {TextField: {"widget": TextInput(attrs={"size": "32"})}}


class IndicatorAdmin(AdminInfoMixin, nested_admin.NestedModelAdmin):
    admin_info = "Paramétrage des indicateurs."
    model = Indicator
    inlines = [DimensionInline]
    list_display = [
        "name",
        "title",
        "sub_theme",
        "index_in_theme",
        "is_active",
    ]

    def save_model(self, request, indicator, form, change):
        super().save_model(request, indicator, form, change)
        if "methodo" in form.changed_data:
            reset_methodo_file(indicator)

    def change_view(self, request, object_id, form_url="", extra_context=None):
        extra_context = extra_context or {}
        extra_context["refresh_url"] = f"../../{object_id}/refresh_filters/"
        return super().change_view(request, object_id, form_url, extra_context)

    def get_urls(self):
        urls = super().get_urls()
        custom_urls = [
            path(
                "<path:object_id>/refresh_filters/",
                self.admin_site.admin_view(self.refresh_filters_view),
                name="refresh_filters",
            ),
        ]
        return custom_urls + urls

    def refresh_filters_view(self, request, object_id):
        indicator = Indicator.objects.get(pk=object_id)
        try:
            for dimension in indicator.dimensions.all():
                refresh_filters(dimension)
            self.message_user(request, "Les filtres ont été actualisés.")
        except Exception as e:
            print(e)
            self.message_user(
                request,
                f"Erreur lors de l'actualisation des filtres: {str(e)}",
                level="error",
            )

        return redirect("../")


class IndicatorInline(admin.TabularInline):
    model = Indicator
    extra = 0
    fields = ["name", "index_in_theme", "link"]
    readonly_fields = ["link"]
    ordering = ["index_in_theme"]

    def link(self, obj):
        url = reverse("admin:indicators_lib_indicator_change", args=[obj.id])
        return format_html(f'<a href="{url}">Edit</a>')


class SubThemesInline(admin.TabularInline):
    model = SubTheme
    extra = 0
    fields = [
        "name",
        "index_in_theme",
        "indicators_count",
        "is_displayed_on_app",
        "link",
    ]
    ordering = ["index_in_theme"]
    readonly_fields = ["indicators_count", "is_displayed_on_app", "link"]

    def link(self, obj):
        url = reverse("admin:indicators_lib_subtheme_change", args=[obj.id])
        return format_html(f'<a href="{url}">Edit</a>')

    link.short_description = "Link"


class ThemeAdmin(AdminInfoMixin, admin.ModelAdmin):
    admin_info = "Paramétrage des thèmes des indicateurs."
    model = Theme
    inlines = [SubThemesInline]
    list_display = [
        "title",
        "ordering",
        "subthemes_count",
        "indicators_count",
        "is_displayed_on_app",
    ]


class SubThemeAdmin(AdminInfoMixin, admin.ModelAdmin):
    admin_info = "Paramétrage des sous-thèmes des indicateurs."
    model = SubTheme
    inlines = [IndicatorInline]
    list_display = [
        "title",
        "theme",
        "index_in_theme",
        "indicators_count",
        "is_displayed_on_app",
    ]


admin.site.register(Theme, ThemeAdmin)
admin.site.register(SubTheme, SubThemeAdmin)
admin.site.register(Indicator, IndicatorAdmin)



# Generated by Django 5.2.10 on 2026-01-19 15:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geo_lib', '0003_geofeature_color_column_geofeature_size_column'),
    ]

    operations = [
        migrations.AddField(
            model_name='geofeature',
            name='legend',
            field=models.TextField(blank=True, help_text='Si les points sur la carte ont une couleur ou une taille différente par point, veuillez indiquer une légende dans ce champ.<br/>\n        Le format est le suivant :<br/>\n        <ul>\n            <li>\n                Chaque ligne correspond à une ligne de la légende.\n            </li>\n            <li>\n                Vous pouvez commencez une ligne avec un icone du point selon une couleur et une taille précise.<br/>\n                Dans ce cas commencez la ligne par le format suivant : "icone(couleur:#000000,taille:sm)"<br/>\n                Utilisez un code couleur hexadécimal, ou bien "couleur:nulle" pour ne pas mettre de couleur.<br/>\n                Les deux valeurs possibles pour la taille sont : "sm" et "md".<br/>\n                <i>Exemple:</i> "icone(couleur:nulle,taille:md) Stationnement avec capacité > 20 places"\n            </li>\n            <li>\n                Vous pouvez commencer une ligne par un carré de couleur.<br/>\n                Le format est : "couleur(#000000)"<br/>\n                <i>Exemple</i> "couleur(#ff00ff) Stationnement payant"\n            </li>\n        </ul>', null=True, verbose_name='Légende'),
        ),
    ]



import re

from django.apps import apps
from django.urls import reverse


def parse_header_navigation(header_navigation):
    """
    Retourne une liste de sections. Chaque section est soit :
    - {'title': 'Accueil', 'link': '/accueil/'}
    - {'title': 'À propos', 'links': [{'title': 'Présentation', 'link': '/presentation/'}]}
    """
    menu = []

    # Découpage robuste en blocs (rubriques), même s'il y a des \r, espaces, etc.
    blocks = re.split(r"\n\s*\n", header_navigation.strip())

    for block in blocks:
        lines = [line.strip() for line in block.strip().splitlines() if line.strip()]
        if not lines:
            continue

        title = None
        links = []

        # Si la première ligne n’est pas un lien markdown, c’est le titre
        first_line = lines[0]
        match = re.match(r"\[(.+?)\]\((/.+?/)\)", first_line)
        if match:
            # Bloc sans titre
            links.append(
                {
                    "title": match.group(1).strip(),
                    "link": match.group(2).strip(),
                }
            )
            other_lines = lines[1:]
        else:
            # cas spécial superset
            if first_line == "superset":
                Dashboard = apps.get_model("superset_lib", "Dashboard")
                links = [
                    {
                        "title": d.label,
                        "link": reverse(
                            "website:superset", kwargs={"dashboard_name": d.short_name}
                        )
                        + "#header-navigation",
                    }
                    for d in Dashboard.objects.all().order_by("order")
                ]
                menu.append({"title": "Portraits de territoires", "links": links})
                continue
            # La première ligne est le titre
            title = first_line
            other_lines = lines[1:]

        # Parse les lignes suivantes comme liens
        for line in other_lines:
            match = re.match(r"\[(.+?)\]\((/.+?/)\)", line)
            if not match:
                raise ValueError(
                    f"La ligne « {line} » n’est pas un lien valide Markdown."
                )
            links.append(
                {
                    "title": match.group(1).strip(),
                    "link": match.group(2).strip(),
                }
            )

        # Construction finale
        if title:
            if not links:
                raise ValueError(f"La rubrique « {title} » ne contient aucun lien.")
            menu.append({"title": title, "links": links})
        elif len(links) == 1:
            menu.append(links[0])
        else:
            raise ValueError("Un groupe de plusieurs liens doit avoir un titre.")
    return menu


def parse_footer_navigation(footer_navigation):
    """
    Retourne une liste de liens : chacun est un dict avec :
    - title : texte à afficher
    - href : URL du lien
    - is_external : True/False
    """
    links = []
    for line in footer_navigation.strip().splitlines():
        line = line.strip()
        if not line:
            continue

        match = re.match(r"\[(.+?)\]\((.+?)\)", line)
        if not match:
            raise ValueError(f"Ligne invalide dans le footer : « {line} »")

        title, href = match.group(1).strip(), match.group(2).strip()

        is_external = href.startswith("http")
        links.append(
            {
                "title": title,
                "href": href,
                "is_external": is_external,
            }
        )
    return links


def parse_markdown_link(text):
    text = text.strip()
    if not text:
        raise ValueError("Le lien est invalide (vide)")

    match = re.match(r"\[(.+?)\]\((.+?)\)", text)
    if not match:
        raise ValueError(f"Ligne invalide dans le footer : « {text} »")

    title, href = match.group(1).strip(), match.group(2).strip()
    return {"title": title, "href": href}



from django.urls import path

from territories_dashboard_lib.tracking_lib.views import track_event_view

app_name = "tracking-api"

urlpatterns = [
    path("event/", track_event_view, name="event"),
]



from django.urls import path

from territories_dashboard_lib.website_lib.views.api import (
    left_menu_view,
)

app_name = "website-api"

urlpatterns = [
    path("left-menu/<str:theme_name>/", left_menu_view, name="left-menu"),
]



from django.shortcuts import render
from django.views.decorators.http import require_GET

from territories_dashboard_lib.website_lib.views.utils import (
    get_theme_and_sub_themes,
    get_themes,
)


@require_GET
def left_menu_view(request, theme_name):
    comparison = request.GET.get("comparison") == "true"
    theme, sub_themes = get_theme_and_sub_themes(request, theme_name)
    themes = get_themes(request)
    context = {
        "theme": theme,
        "themes": themes,
        "sub_themes": sub_themes,
        "comparison": comparison,
    }
    if request.user.is_authenticated and request.user.favorite_indicators.count() == 1:
        context["new_favorite"] = True
    return render(
        request,
        "territories_dashboard_lib/website/pages/indicators/components/themes-nav.html",
        context,
    )



from .payloads import IndicatorTablePayload
from .query.commons import (
    add_optional_filters,
    get_values_for_indicator_territory_submesh,
)
from .query.utils import run_custom_query


def _get_query(
    *, indicator, props: IndicatorTablePayload, columns, orders, filters, limit=None
):
    columns = [c for c in columns if c is not None]
    orders = [o for o in orders if o is not None]
    order_clause = f"ORDER BY {', '.join(orders)}" if len(orders) > 0 else ""
    limitation = f"LIMIT {limit}" if limit is not None else ""
    offset = f"OFFSET {(props.pagination - 1) * limit}" if limit is not None else ""
    dimension_search_where = (
        f"OR (unaccent(LOWER({indicator.flows_dimension})) LIKE unaccent(LOWER(CONCAT('%%', %(search)s, '%%'))))"
        if props.flows
        else " ".join(
            [
                f"OR (unaccent(LOWER({dimension.db_name})) LIKE unaccent(LOWER(CONCAT('%%', %(search)s, '%%'))))"
                for dimension in indicator.dimensions.all()
            ]
        )
    )
    territory_search_where = (
        "OR (unaccent(LOWER(arbo1.name)) LIKE unaccent(LOWER(CONCAT('%%', %(search)s, '%%')))) OR (unaccent(LOWER(arbo2.name)) LIKE unaccent(LOWER(CONCAT('%%', %(search)s, '%%'))))"
        if props.flows
        else (
            "OR (unaccent(LOWER(name)) LIKE unaccent(LOWER(CONCAT('%%', %(search)s, '%%'))))"
        )
    )
    where = (
        f"""
        AND (
            (LENGTH(TRIM(%(search)s)) = 0)
            {territory_search_where}
            {dimension_search_where}
            OR (CAST(annee AS TEXT) LIKE CONCAT('%%', %(search)s, '%%'))
        )
    """
        if props.search
        else ""
    )
    where_year = f"AND annee = {props.year}" if props.year else ""
    regular_dimensions_filters = (
        "" if props.flows else add_optional_filters(indicator, filters)
    )
    return f"""
        SELECT
            {", ".join(columns)}
        FROM
            {get_values_for_indicator_territory_submesh(indicator=indicator, territory=props.territory, submesh=props.submesh, flows=props.flows)}
            {regular_dimensions_filters}
        {where} {where_year}
        {order_clause}
        {f"{limitation} {offset}"}
    """


def get_values_columns(indicator, props, *, for_export=False):
    if props.flows:
        columns = [
            "annee",
            "arbo1.name as territory_1",
            "arbo2.name as territory_2",
            f"{indicator.flows_dimension} AS dimension"
            if indicator.flows_dimension
            else None,
            "CAST(valeur AS int) as valeur",
        ]
        if for_export:
            columns += ["arbo1.code as territory_1_id", "arbo2.code as territory_2_id"]
    else:
        columns = (
            ["annee", "name"]
            + [f"{dimension.db_name}" for dimension in indicator.dimensions.all()]
            + [
                f"valeur * {indicator.aggregation_constant if indicator.unite == '%' else 1} AS valeur",
                "CASE WHEN valeur IS NULL THEN NULL ELSE composante_1 END AS valeur_alternative"
                if indicator.is_composite and indicator.show_alternative
                else None,
            ]
        )
        if for_export:
            columns += ["code"]
    return columns


def get_values_orders(indicator, props: IndicatorTablePayload):
    territory_order = None
    if props.column_order not in ["name", "arbo1.name", "arbo2.name"]:
        territory_order = (
            "LOWER(arbo1.name) ASC, LOWER(arbo2.name) ASC"
            if props.flows
            else "LOWER(name) ASC"
        )
    year_order = "annee DESC" if props.column_order != "annee" else None
    dimension_order = None
    if props.flows:
        dimension = indicator.flows_dimension
        if props.column_order != dimension:
            dimension_order = f"{dimension} DESC"
    else:
        dimensions = [dimension.db_name for dimension in indicator.dimensions.all()]
        if props.column_order not in [dimensions]:
            dimension_order = (
                ", ".join([f"{dimension} DESC" for dimension in dimensions])
                if dimensions
                else None
            )
    orders = [
        f"{props.column_order} {props.column_order_flow} NULLS LAST"
        if props.column_order
        else None,
        year_order,
        territory_order,
        dimension_order,
    ]
    return orders


def get_count_and_data_for_indicator_table(
    indicator, props: IndicatorTablePayload, filters
):
    count_query = _get_query(
        indicator=indicator,
        props=props,
        columns=["COUNT(*)"],
        orders=[],
        filters=filters,
    )
    count = run_custom_query(count_query, {"search": props.search})
    columns = get_values_columns(indicator, props)
    orders = get_values_orders(indicator, props)
    data_query = _get_query(
        indicator=indicator,
        props=props,
        columns=columns,
        orders=orders,
        limit=props.limit,
        filters=filters,
    )
    data = run_custom_query(data_query, {"search": props.search})
    return count[0]["count"], data


def get_export_indicator_table_values(indicator, props: IndicatorTablePayload, filters):
    query = _get_query(
        indicator=indicator,
        props=props,
        columns=get_values_columns(indicator, props, for_export=True),
        orders=get_values_orders(indicator, props),
        filters=filters,
    )
    return run_custom_query(query, {"search": props.search})



from django.contrib.auth import get_user_model

DefaultUserModel = get_user_model()


class ProxyUser(DefaultUserModel):
    class Meta:
        proxy = True

    @property
    def has_favorite_indicators(self):
        return self.favorite_indicators.exists()



# Generated by Django 5.2.11 on 2026-02-04 18:23

import django.db.models.deletion
import martor.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('indicators_lib', '0006_indicator_hidden'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='subtheme',
            name='description',
        ),
        migrations.AlterField(
            model_name='filter',
            name='color',
            field=models.TextField(blank=True, help_text='format hexadécimale : #FF11DD. Les données associées à ce filtre apparaitront de cette couleur dans les graphiques.', null=True, verbose_name='Couleur'),
        ),
        migrations.AlterField(
            model_name='filter',
            name='db_name',
            field=models.CharField(max_length=128, verbose_name='Nom dans la BDD'),
        ),
        migrations.AlterField(
            model_name='filter',
            name='default',
            field=models.BooleanField(default=True, verbose_name='Sélectionné par défaut ?'),
        ),
        migrations.AlterField(
            model_name='filter',
            name='order',
            field=models.IntegerField(default=0, help_text="L'ordre démarre à 0.", verbose_name='Ordre'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='aggregation_constant',
            field=models.DecimalField(decimal_places=5, default=1, max_digits=10, verbose_name="Constante d'agrégation"),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='aggregation_function',
            field=models.CharField(choices=[('discrete', 'Discrete Component 2'), ('repeated', 'Repeated Component 2')], default='repeated', max_length=8, verbose_name="Fonction d'agrégation"),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='db_table_prefix',
            field=models.CharField(help_text='Attention, le nom doit être exactement le même que celui en base.', max_length=128, verbose_name='Préfixe dans la DB'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='flows_db_table_prefix',
            field=models.CharField(blank=True, help_text='Uniquement pour les indicateurs de flux. Nom de la table des flux.', max_length=128, null=True, verbose_name='Table des flux (prefix)'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='flows_dimension',
            field=models.CharField(blank=True, help_text='Uniquement pour les indicateurs de flux. Nom de la colonne de dimension dans la table des flux.', max_length=32, null=True, verbose_name='Dimension des flux'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='index_in_theme',
            field=models.IntegerField(default=0, help_text="L'ordre démarre à 0.", verbose_name='Ordre sous-thème'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='Actif ?'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='is_composite',
            field=models.BooleanField(default=False, verbose_name='Indicateur composite'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='methodo',
            field=martor.models.MartorField(blank=True, default='', verbose_name='Méthodologie (markdown)'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='name',
            field=models.CharField(help_text="Pour l'URL, ne mettre que des lettres minuscules sans accents et des tirets.", max_length=32, unique=True, verbose_name='Nom (~id, URL)'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='secondary_indicator',
            field=models.ForeignKey(blank=True, help_text='Optionnel.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='indicators_lib.indicator', verbose_name='Second indicateur à afficher sur la carte'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='short_title',
            field=models.CharField(blank=True, help_text='Utilisé pour les popups et la légende de la carte.', max_length=128, null=True, verbose_name='Titre abrégé'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='show_alternative',
            field=models.BooleanField(default=True, help_text="Pour certains indicateurs, il n'y a pas de valeur alternative à afficher.", verbose_name='Afficher la valeur alternative'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='show_evolution',
            field=models.BooleanField(default=True, verbose_name="Activer l'historique"),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='sub_theme',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='indicators', to='indicators_lib.subtheme', verbose_name='Sous-thème'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='title',
            field=models.CharField(max_length=128, verbose_name='Titre (affiché)'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='unite',
            field=models.CharField(max_length=32, verbose_name='Unité (affichée)'),
        ),
        migrations.AlterField(
            model_name='indicator',
            name='unite_alternative',
            field=models.CharField(blank=True, default=None, max_length=32, null=True, verbose_name='Unité alternative (affichée)'),
        ),
        migrations.AlterField(
            model_name='subtheme',
            name='index_in_theme',
            field=models.IntegerField(default=0, help_text="L'ordre démarre à 0.", verbose_name='Ordre thème'),
        ),
        migrations.AlterField(
            model_name='subtheme',
            name='name',
            field=models.CharField(help_text="Pour l'URL, ne mettre que des lettres minuscules sans accents et des tirets.", max_length=64, unique=True, verbose_name='Nom (~id, URL)'),
        ),
        migrations.AlterField(
            model_name='subtheme',
            name='theme',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='sub_themes', to='indicators_lib.theme', verbose_name='Thème'),
        ),
        migrations.AlterField(
            model_name='subtheme',
            name='title',
            field=models.CharField(max_length=128, verbose_name='Titre (affiché)'),
        ),
        migrations.AlterField(
            model_name='theme',
            name='action_theme',
            field=models.TextField(blank=True, verbose_name='Actions'),
        ),
        migrations.AlterField(
            model_name='theme',
            name='name',
            field=models.CharField(help_text="Pour l'URL, ne mettre que des lettres minuscules sans accents et des tirets.", max_length=64, unique=True, verbose_name='Nom (~id, URL)'),
        ),
        migrations.AlterField(
            model_name='theme',
            name='objectif_theme',
            field=models.TextField(blank=True, verbose_name='Objectif'),
        ),
        migrations.AlterField(
            model_name='theme',
            name='ordering',
            field=models.IntegerField(default=0, help_text="L'ordre démarre à 0.", verbose_name='Ordre dans la sidebar'),
        ),
        migrations.AlterField(
            model_name='theme',
            name='title',
            field=models.CharField(max_length=128, verbose_name='Titre (affiché)'),
        ),
    ]



# Generated by Django 5.2.11 on 2026-02-10 16:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('superset_lib', '0003_alter_dashboard_order_alter_filter_mesh_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='filter',
            name='mesh',
            field=models.TextField(blank=True, choices=[('fr', 'Fr'), ('reg', 'Reg'), ('dep', 'Dep'), ('epci', 'Epci'), ('com', 'Com'), ('aom', 'Aom'), ('ept', 'Ept')], help_text="Maille pour laquelle s'effectue le filtre sur le territoire.", null=True),
        ),
    ]



# Generated by Django 5.2.9 on 2026-01-06 16:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracking_lib', '0005_alter_page_cmp_territory_mesh_alter_page_submesh_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='name',
            field=models.TextField(choices=[('download', 'Download'), ('vue-resume-indicateur', 'View Indicator Card')]),
        ),
    ]



from django.db import models

from territories_dashboard_lib.commons.models import CommonModel
from territories_dashboard_lib.indicators_lib.models import Indicator

from .enums import GeoFeatureType


class GeoFeature(CommonModel):
    name = models.TextField(
        verbose_name="Nom de la table en base",
        help_text="Attention, le nom doit être exactement le même que celui en base.",
    )
    title = models.TextField(verbose_name="Titre à afficher", default="à compléter")
    unite = models.TextField(verbose_name="Unité au pluriel")
    indicator = models.ForeignKey(
        Indicator,
        related_name="geo_features",
        on_delete=models.CASCADE,
        help_text="Cette donnée géographique sera ajoutée à la carte de l'indicateur.",
    )
    geo_type = models.TextField(
        choices=GeoFeatureType.choices,
        default=GeoFeatureType.point,
        verbose_name="Type de géométrie",
    )
    point_icon_svg = models.TextField(
        blank=True,
        null=True,
        verbose_name="Icone des points",
        help_text="Icone SVG à utiliser pour les points.<br/>Laisser vide pour les autres types de géométrie.<br/>Veuillez charger un fichier provenant des <a href='https://fonts.google.com/icons'>Material Icons</a> de Google.",
    )
    color = models.TextField(
        default="#000000",
        verbose_name="Couleur",
        help_text="Couleur des données sur la carte, à renseigner au format hexadécimal, par exemple: '#05B2F9'",
    )
    show_on_fr_level = models.BooleanField(
        default=True,
        verbose_name="Afficher au niveau France",
        help_text="Décochez si la quantité de données est trop importante pour le niveau France.",
    )
    color_column = models.TextField(
        blank=True,
        null=True,
        verbose_name="Colonne de la couleur",
        help_text="Optionnel. Nom de la colonne qui définit une couleur par point. Les valeurs de la colonne doivent être sous le format #ffffff (hexadecimale). Si renseigné, le champ 'color' ne sera pas utilisé. Seulement pour les points.",
    )
    size_column = models.TextField(
        blank=True,
        null=True,
        verbose_name="Colonne de la taille",
        help_text="Optionnel. Nom de la colonne qui définit une taille par point. Les valeurs de la colonne doivent être 'sm' (small) ou 'md' (medium). Seulement pour les points.",
    )
    legend = models.TextField(
        blank=True,
        null=True,
        verbose_name="Légende",
        help_text="""Si les points sur la carte ont une couleur ou une taille différente par point, veuillez indiquer une légende dans ce champ.<br/>
        Le format est le suivant :<br/>
        <ul>
            <li>
                Chaque ligne correspond à une ligne de la légende.
            </li>
            <li>
                Vous pouvez commencez une ligne avec un icone du point selon une couleur et une taille précise.<br/>
                Dans ce cas commencez la ligne par le format suivant : "icone(couleur:#000000,taille:sm)"<br/>
                Utilisez un code couleur hexadécimal, ou bien "couleur:nulle" pour ne pas mettre de couleur.<br/>
                Les deux valeurs possibles pour la taille sont : "sm" et "md".<br/>
                <i>Exemple:</i> "icone(couleur:nulle,taille:md) Stationnement avec capacité > 20 places"
            </li>
            <li>
                Vous pouvez commencer une ligne par un carré de couleur.<br/>
                Le format est : "couleur(#000000)"<br/>
                <i>Exemple</i> "couleur(#ff00ff) Stationnement payant"
            </li>
        </ul>""",
    )

    def __str__(self):
        return self.name

    class Meta:
        ordering = ("indicator", "name")
        verbose_name = "Données de la carte"
        verbose_name_plural = "Données de la carte"


class GeoElement(CommonModel):
    geo_feature = models.ForeignKey(
        GeoFeature, related_name="items", on_delete=models.CASCADE
    )
    name = models.TextField(verbose_name="Nom de la colonne en DB")
    label = models.TextField()
    filterable = models.BooleanField(
        default=True,
        verbose_name="Filtrable",
        help_text="Si coché, l'attribut est filtrable par l'utilisateur dans le volet de gestion des filtres.",
    )
    linked_to_indicator = models.BooleanField(
        default=False,
        verbose_name="Relié aux filtres de l'indicateur",
        help_text="Si coché, le filtre de l'indicateur avec le même nom sera relié au filtre de la carte.",
    )



# Generated by Django 5.2.11 on 2026-02-10 16:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracking_lib', '0007_alter_event_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='page',
            name='cmp_territory_mesh',
            field=models.TextField(blank=True, choices=[('fr', 'Fr'), ('reg', 'Reg'), ('dep', 'Dep'), ('epci', 'Epci'), ('com', 'Com'), ('aom', 'Aom'), ('ept', 'Ept')], help_text="Maille du territoire de comparaison, null s'il ne s'agit pas de la page comparaison.", null=True),
        ),
        migrations.AlterField(
            model_name='page',
            name='submesh',
            field=models.TextField(blank=True, choices=[('fr', 'Fr'), ('reg', 'Reg'), ('dep', 'Dep'), ('epci', 'Epci'), ('com', 'Com'), ('aom', 'Aom'), ('ept', 'Ept')], help_text="Maille d'analyse sélectionnée.", null=True),
        ),
        migrations.AlterField(
            model_name='page',
            name='territory_mesh',
            field=models.TextField(blank=True, choices=[('fr', 'Fr'), ('reg', 'Reg'), ('dep', 'Dep'), ('epci', 'Epci'), ('com', 'Com'), ('aom', 'Aom'), ('ept', 'Ept')], help_text='Maille du territoire principal sélectionné.', null=True),
        ),
    ]



from django.http import HttpResponse, JsonResponse
from django.views.decorators.http import require_GET

from territories_dashboard_lib.commons.decorators import use_payload
from territories_dashboard_lib.superset_lib.payloads import GuestTokenPayload

from .guest_token import get_guest_token
from .models import Dashboard


@require_GET
@use_payload(GuestTokenPayload)
def guest_token_view(request, payload):
    guest_token = get_guest_token(payload.dashboard)
    return HttpResponse(guest_token)


@require_GET
def dashboards_view(request):
    dashboards = Dashboard.objects.order_by("order", "label")
    data = [{"label": d.label, "superset_id": d.superset_id} for d in dashboards]
    return JsonResponse(data, safe=False)



from django.urls import reverse

from territories_dashboard_lib.website_lib.models import StaticPage


def get_website_links():

    links = []
    links.append(reverse("website:landing-page"))
    links.append(reverse("website:indicateurs"))
    links.append(reverse("website:comparison-redirect"))
    links.append(reverse("website:lexique"))
    links.append(reverse("website:sitemap"))
    for static_page in StaticPage.objects.all():
        links.append(
            reverse("website:static-page", kwargs={"page_url": static_page.url})
        )
    return links



# Generated by Django 5.2.11 on 2026-02-04 18:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('superset_lib', '0002_alter_filter_mesh'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dashboard',
            name='order',
            field=models.IntegerField(default=0, help_text="Numéro d'ordre dans la dropdown de sélection, les dashboards sont triés du plus petit numéro au plus grand. L'ordre démarre à 0.", verbose_name="Numéro d'ordre"),
        ),
        migrations.AlterField(
            model_name='filter',
            name='mesh',
            field=models.TextField(blank=True, choices=[('fr', 'Fr'), ('reg', 'Reg'), ('dep', 'Dep'), ('epci', 'Epci'), ('com', 'Com'), ('aom', 'Aom')], help_text="Maille pour laquelle s'effectue le filtre sur le territoire.", null=True),
        ),
        migrations.AlterField(
            model_name='filter',
            name='superset_col',
            field=models.TextField(blank=True, help_text="Nom de la colonne en base de données liée au filtre sur le territoire et utilisé dans Superset. Pour trouver le nom, on peut faire les mêmes étapes que pour le 'superset_id' et chercher 'column' dans les paramètres json du native filter.", null=True),
        ),
        migrations.AlterField(
            model_name='filter',
            name='superset_id',
            field=models.TextField(blank=True, help_text="ID du native filter du territoire, qui permettra d'initialiser les filtres du dasbhoard selon le territoire sélectionné dans l'application. Pour le récupérer c'est un petit parcours du combattant : aller sur le dashboard dans superset, cliquer sur modifier le dashboard, cliquer sur les trois petits points, puis sur modifier les propriétés. Cliquer sur avancé, copier le json et le coller dans un site comme : https://jsonformatter.curiousconcept.com/ pour mieux le voir. Chercher 'global_chart_configuration' puis 'native_filter_configuration'. Chercher le native filter lié au choix du territoire. Copier l'id qui dans son nom, le nom est de la forme NATIVE_FILTER-ID. ", null=True),
        ),
    ]



from django.http import HttpResponse
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse
from django.views.decorators.gzip import gzip_page

from territories_dashboard_lib.indicators_lib.models import Indicator, Theme
from territories_dashboard_lib.superset_lib.logic import make_filter
from territories_dashboard_lib.superset_lib.models import Dashboard
from territories_dashboard_lib.superset_lib.serializers import serialize_dashboard
from territories_dashboard_lib.tracking_lib.logic import track_page
from territories_dashboard_lib.website_lib.logic import (
    get_favorite_theme,
)
from territories_dashboard_lib.website_lib.models import (
    GlossaryItem,
    LandingPage,
    StaticPage,
)
from territories_dashboard_lib.website_lib.params import with_params
from territories_dashboard_lib.website_lib.serializers import (
    serialize_indicator,
    serialize_sub_themes,
)
from territories_dashboard_lib.website_lib.views.utils import (
    apply_filters,
    get_base_url,
    get_favorite_indicators_ids,
    get_theme_and_sub_themes,
    get_themes,
)


@gzip_page
def landing_page_view(request):
    landing_page = LandingPage.objects.first()
    response = render(
        request,
        "territories_dashboard_lib/website/pages/page.html",
        {"landing_page": landing_page},
    )
    response = track_page(request=request, response=response)
    return response


@gzip_page
def static_page_view(request, page_url):
    static_page = get_object_or_404(StaticPage, url=page_url)
    return render(
        request,
        "territories_dashboard_lib/website/pages/static/page.html",
        {"static_page": static_page},
    )


@gzip_page
def lexique_page_view(request):
    lexique = GlossaryItem.objects.all()
    context = {"lexique": lexique}
    response = render(
        request, "territories_dashboard_lib/website/pages/lexique/page.html", context
    )
    return response


def themes_redirect_view(request):
    first_theme = Theme.objects.order_by("ordering").first()
    if getattr(request.user, "has_favorite_indicators", None):
        first_theme = get_favorite_theme()
    target_url = reverse("website:theme", kwargs={"theme_name": first_theme.name})
    get_params = request.GET.urlencode()
    if get_params:
        target_url = f"{target_url}?{get_params}"
    return redirect(target_url)


@gzip_page
@with_params
def comparison_view(request, *, theme_name, context):
    theme, sub_themes = get_theme_and_sub_themes(request, theme_name)
    themes = get_themes(request)
    context["theme"] = theme
    context["themes"] = themes
    context["sub_themes"] = sub_themes
    context["favorite_indicators_ids"] = get_favorite_indicators_ids(request)
    response = render(
        request,
        "territories_dashboard_lib/website/pages/indicators/comparaison/[theme]/page.html",
        context,
    )
    response = track_page(
        request=request,
        response=response,
        params=context["params"],
        theme=theme,
    )
    return response


def comparison_redirect_view(request):
    first_theme = Theme.objects.order_by("ordering").first()
    if getattr(request.user, "has_favorite_indicators", None):
        first_theme = get_favorite_theme()
    target_url = reverse("website:comparison", kwargs={"theme_name": first_theme.name})
    get_params = request.GET.urlencode()
    if get_params:
        target_url = f"{target_url}?{get_params}"
    return redirect(target_url)


@gzip_page
@with_params
def theme_page_view(request, *, theme_name, context):
    theme, sub_themes = get_theme_and_sub_themes(request, theme_name)
    themes = get_themes(request)
    context["theme"] = theme
    context["themes"] = themes
    context["sub_themes"] = sub_themes
    context["favorite_indicators_ids"] = get_favorite_indicators_ids(request)
    response = render(
        request,
        "territories_dashboard_lib/website/pages/indicators/themes/page.html",
        context,
    )
    response = track_page(
        request=request,
        response=response,
        params=context["params"],
        theme=theme,
    )
    return response


@gzip_page
@with_params
def indicator_details_view(request, *, indicator_name, context):
    indicator = get_object_or_404(Indicator, name=indicator_name)
    theme = indicator.sub_theme.theme
    sub_themes = serialize_sub_themes(theme)
    themes = get_themes(request)
    filters = apply_filters(request, indicator)
    context["indicator"] = serialize_indicator(indicator)
    context["themes"] = themes
    context["theme"] = theme
    context["subtheme"] = indicator.sub_theme
    context["sub_themes"] = sub_themes
    context["filters"] = filters
    response = render(
        request,
        "territories_dashboard_lib/website/pages/indicators/details/page.html",
        context=context,
    )
    response = track_page(
        request=request,
        response=response,
        params=context["params"],
        indicator=indicator,
        theme=theme,
    )
    return response


@gzip_page
def indicator_methodo_view(request, *, indicator_name):
    indicator = get_object_or_404(Indicator, name=indicator_name)
    theme = indicator.sub_theme.theme
    context = {
        "indicator": serialize_indicator(indicator),
        "theme": theme,
    }
    response = render(
        request,
        "territories_dashboard_lib/website/pages/indicators/methodo/page.html",
        context,
    )
    response = track_page(
        request=request,
        response=response,
        indicator=indicator,
        theme=theme,
    )
    return response


@with_params
def superset_view(request, dashboard_name, context):
    dashboard = get_object_or_404(Dashboard, short_name=dashboard_name)
    context["dashboard"] = serialize_dashboard(dashboard)
    filter_string = make_filter(
        dashboard,
        context["params"]["territory_id"],
        context["params"]["territory_mesh"],
    )
    if filter_string:
        context["dashboard_filter"] = filter_string
    response = render(
        request,
        "territories_dashboard_lib/website/pages/superset/page.html",
        context=context,
    )
    response = track_page(request=request, response=response)
    return response


def handler_404_view(request, exception):
    return render(request, "territories_dashboard_lib/website/404.html", status=404)


def handler_500_view(request):
    return render(request, "territories_dashboard_lib/website/500.html", status=500)


@gzip_page
def sitemap_view(request):
    context = {
        "themes": [
            {"title": theme.title, "name": theme.name}
            for theme in Theme.objects.all().order_by("ordering")
        ],
        "indicators": [
            {"title": indicator.title, "name": indicator.name}
            for indicator in Indicator.objects.all().order_by("index_in_theme")
        ],
        "static_pages": [
            {"title": static_page.name, "url": static_page.url}
            for static_page in StaticPage.objects.all()
        ],
        "lexique": GlossaryItem.objects.exists(),
    }
    response = render(
        request, "territories_dashboard_lib/website/pages/sitemap/page.html", context
    )
    response = track_page(request=request, response=response)
    return response


def raw_sitemap_view(request):
    """Generate XML sitemap for search engines."""
    base_url = get_base_url(request)

    urls = []

    # Landing page
    urls.append(f"{base_url}{reverse('website:landing-page')}")

    # Theme pages
    for theme in Theme.objects.all().order_by("ordering"):
        urls.append(
            f"{base_url}{reverse('website:theme', kwargs={'theme_name': theme.name})}"
        )

    # Comparison pages
    for theme in Theme.objects.all().order_by("ordering"):
        urls.append(
            f"{base_url}{reverse('website:comparison', kwargs={'theme_name': theme.name})}"
        )

    # Indicator detail pages
    for indicator in Indicator.objects.all().order_by("index_in_theme"):
        urls.append(
            f"{base_url}{reverse('website:indicator-details', kwargs={'indicator_name': indicator.name})}"
        )
        # Indicator methodology pages
        urls.append(
            f"{base_url}{reverse('website:indicator-methodo', kwargs={'indicator_name': indicator.name})}"
        )

    # Static pages
    for static_page in StaticPage.objects.all():
        urls.append(
            f"{base_url}{reverse('website:static-page', kwargs={'page_url': static_page.url})}"
        )

    # Lexique page (if glossary items exist)
    if GlossaryItem.objects.exists():
        urls.append(f"{base_url}{reverse('website:lexique')}")

    # Sitemap page itself
    urls.append(f"{base_url}{reverse('website:sitemap')}")

    # Generate XML
    xml_lines = [
        '<?xml version="1.0" encoding="UTF-8"?>',
        '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">',
    ]

    for url in urls:
        xml_lines.append("  <url>")
        xml_lines.append(f"    <loc>{url}</loc>")
        xml_lines.append("  </url>")

    xml_lines.append("</urlset>")

    xml_content = "\n".join(xml_lines)

    return HttpResponse(xml_content, content_type="application/xml")


def robots_txt_view(request):
    base_url = get_base_url(request)
    text_content = f"User-agent: *\nSitemap: {base_url}/sitemap.xml"
    return HttpResponse(text_content, content_type="text/plain")



# Generated by Django 5.2.11 on 2026-03-03 16:11

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('indicators_lib', '0008_alter_indicator_min_mesh'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='UserFavoriteIndicator',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveIntegerField(default=0)),
                ('indicator', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='favorite_links', to='indicators_lib.indicator')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='favorite_indicator_links', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('user', 'order'),
                'unique_together': {('user', 'indicator')},
            },
        ),
        migrations.AddField(
            model_name='indicator',
            name='favorited_by',
            field=models.ManyToManyField(related_name='favorite_indicators', through='indicators_lib.UserFavoriteIndicator', to=settings.AUTH_USER_MODEL),
        ),
    ]



from django.db.models import Count, Q
from martor.utils import markdownify

from territories_dashboard_lib.geo_lib.logic import parse_legend


def serialize_indicator(indicator):
    return {
        "id": indicator.id,
        "name": indicator.name,
        "title": indicator.title,
        "short_title": indicator.short_title,
        "description": indicator.description,
        "source": indicator.source,
        "unite": indicator.unite,
        "unite_nom_accessible": indicator.unite_nom_accessible,
        "unite_alternative": indicator.unite_alternative,
        "unite_alternative_nom_accessible": indicator.unite_alternative_nom_accessible,
        "show_alternative": indicator.show_alternative,
        "methodo_html": markdownify(indicator.methodo),
        "db_table_prefix": indicator.db_table_prefix,
        "is_composite": indicator.is_composite,
        "aggregation_constant": indicator.aggregation_constant,
        "flows_db_table_prefix": indicator.flows_db_table_prefix,
        "flows_dimension": indicator.flows_dimension,
        "theme": indicator.sub_theme.theme.title,
        "filters": serializer_filters(indicator),
        "show_evolution": indicator.show_evolution,
        "min_mesh": indicator.min_mesh,
        "secondary_indicator": {
            "name": indicator.secondary_indicator.name,
            "title": indicator.secondary_indicator.title,
            "short_title": indicator.secondary_indicator.short_title,
            "aggregation_constant": indicator.secondary_indicator.aggregation_constant,
            "db_table_prefix": indicator.secondary_indicator.db_table_prefix,
            "unite": indicator.secondary_indicator.unite,
            "unite_alternative": indicator.secondary_indicator.unite_alternative,
            "is_composite": indicator.secondary_indicator.is_composite,
            "filters": serializer_filters(indicator.secondary_indicator),
        }
        if indicator.secondary_indicator
        else None,
        "geo_features": [
            {
                "id": gf.id,
                "name": gf.name,
                "title": gf.title,
                "unite": gf.unite,
                "geo_type": gf.geo_type,
                "point_icon_svg": gf.point_icon_svg,
                "color": gf.color,
                "show_on_fr_level": gf.show_on_fr_level,
                "legend": parse_legend(gf.legend),
                "items": [
                    {
                        "name": item.name,
                        "label": item.label,
                        "filterable": item.filterable,
                        "linked_to_indicator": item.linked_to_indicator,
                    }
                    for item in gf.items.all()
                ],
            }
            for gf in indicator.geo_features.all()
        ],
    }


def serializer_filters(indicator):
    return [
        (
            {
                "db_name": dimension.db_name,
                "title": dimension.title,
                "is_breakdown": dimension.is_breakdown,
            },
            [
                {
                    "db_name": filtr.db_name,
                    "default": filtr.default,
                }
                for filtr in dimension.filters.all()
            ],
        )
        for dimension in indicator.dimensions.all()
    ]


def filter_indicators(queryset):
    return queryset.filter(is_active=True, hidden=False).order_by("index_in_theme")


def serialize_sub_themes(theme):
    sub_themes = (
        theme.sub_themes.all()
        .annotate(
            active_indicators_count=Count(
                "indicators",
                filter=Q(indicators__is_active=True, indicators__hidden=False),
            )
        )
        .filter(active_indicators_count__gt=0)
        .order_by("index_in_theme")
    )
    sub_themes = [
        {
            "name": sub_theme.name,
            "title": sub_theme.title,
            "indicators": [
                serialize_indicator(indicator)
                for indicator in filter_indicators(sub_theme.indicators)
            ],
        }
        for sub_theme in sub_themes
    ]
    return sub_themes



from typing import List

from django.conf import settings
from django.db import models


class AggregationFunctions(models.TextChoices):
    DISCRETE_COMPONENT_2 = "discrete"
    REPEATED_COMPONENT_2 = "repeated"


class MeshLevel(models.TextChoices):
    fr = "fr"
    reg = "reg"
    dep = "dep"
    epci = "epci"
    com = "com"
    aom = "aom"
    ept = "ept"


STANDARD_MESHES = [
    MeshLevel.fr,
    MeshLevel.reg,
    MeshLevel.dep,
    MeshLevel.epci,
    MeshLevel.com,
]

ALL_MESHES_ABSOLUTE = [
    MeshLevel.fr,
    MeshLevel.reg,
    MeshLevel.dep,
    MeshLevel.aom,
    MeshLevel.epci,
    MeshLevel.ept,
    MeshLevel.com,
]


MESHES_ORDERED_FOR_PRESENTATION = [
    MeshLevel.fr,
    MeshLevel.reg,
    MeshLevel.dep,
    MeshLevel.epci,
    MeshLevel.com,
    MeshLevel.aom,
    MeshLevel.ept,
]


def order_meshes_for_presentation(meshes: List[MeshLevel]) -> List[MeshLevel]:
    """
    Orders a list of MeshLevel values according to MESHES_ORDERED_FOR_PRESENTATION.
    """
    order_index = {mesh: i for i, mesh in enumerate(MESHES_ORDERED_FOR_PRESENTATION)}
    return sorted(meshes, key=lambda mesh: order_index[mesh])


class FranceGeoLevel(models.TextChoices):
    All = "FR0,FR1,FR2"
    METRO = "FR0,FR1"
    METRO_HORS_IDF = "FR0"


FRANCE_GEOLEVEL_TITLES = {
    FranceGeoLevel.All: "France entière",
    FranceGeoLevel.METRO: "France métropolitaine",
    FranceGeoLevel.METRO_HORS_IDF: "France métropolitaine hors IDF",
}

FRANCE_DB_VALUES = {
    FranceGeoLevel.All: "FR_TOT",
    FranceGeoLevel.METRO: "FR_METRO",
    FranceGeoLevel.METRO_HORS_IDF: "FR_METRO_HORS_IDF",
}


DEFAULT_MESH = MeshLevel.reg

MESHES_SHORT_TITLES = {
    MeshLevel.fr: "France",
    MeshLevel.reg: "Région",
    MeshLevel.dep: "Département",
    MeshLevel.epci: "Intercommunalité",
    MeshLevel.com: "Commune",
    MeshLevel.aom: "AOM",
    MeshLevel.ept: "EPT",
}

MESHES_LONG_TITLES = {
    MeshLevel.fr: "France",
    MeshLevel.reg: "Région",
    MeshLevel.dep: "Département",
    MeshLevel.epci: "Intercommunalité",
    MeshLevel.com: "Commune",
    MeshLevel.aom: "Autorité Organisatrice de la Mobilité",
    MeshLevel.ept: "Établissement Public Territorial",
}


def get_miminum_mesh():
    try:
        town_mesh_is_disabled = settings.DISABLE_TOWN_MESH
        return MeshLevel.epci if town_mesh_is_disabled else MeshLevel.com
    except AttributeError:
        return MeshLevel.com


def get_allow_same_mesh():
    try:
        return bool(settings.ALLOW_SAME_MESH)
    except AttributeError:
        return False



import re


def parse_legend(legend: str | None) -> list[dict] | None:
    """
    Parse a legend text field into a structured JSON format.

    Args:
        legend: The legend text, where each line can optionally start with
                "icone(couleur:#XXXXXX,taille:sm|md)" followed by text.

    Returns:
        None if legend is None or empty.
        A list of dicts with keys: color (str|None), size (str|None), text (str).
    """
    if legend is None or legend.strip() == "":
        return None

    result = []
    icon_pattern = re.compile(
        r"^icone\(couleur:(?P<color>#[0-9a-fA-F]{6}|nulle),taille:(?P<size>sm|md)\)\s*"
    )
    color_pattern = re.compile(r"^couleur\((?P<color>#[0-9a-fA-F]{6})\)\s*")

    for line in legend.strip().split("\n"):
        line = line.strip()
        if not line:
            continue

        icon_match = icon_pattern.match(line)
        color_match = color_pattern.match(line)

        color = None
        icon = False
        size = None
        text = line
        if icon_match:
            icon = True
            color = icon_match.group("color")
            color = None if color == "nulle" else color
            size = icon_match.group("size")
            text = line[icon_match.end() :].strip()
        elif color_match:
            color = color_match.group("color")
            text = line[color_match.end() :].strip()
        result.append({"icon": icon, "color": color, "size": size, "text": text})

    return result if result else None



# Generated by Django 5.2.10 on 2026-01-20 10:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('indicators_lib', '0005_auto_20251203_1621'),
    ]

    operations = [
        migrations.AddField(
            model_name='indicator',
            name='hidden',
            field=models.BooleanField(default=False, help_text="Coché pour cacher un indicateur actif, il pourra être utilisé en tant que second indicateur sur la carte mais il n'apparaitra pas dans la liste des indicateurs", verbose_name='Caché'),
        ),
    ]



import json

from django.contrib.auth.decorators import login_required
from django.http import Http404, HttpResponse, JsonResponse
from django.shortcuts import render
from django.template import loader
from django.views.decorators.cache import cache_control
from django.views.decorators.http import require_GET, require_POST

from territories_dashboard_lib.commons.decorators import use_payload
from territories_dashboard_lib.tracking_lib.enums import EventType, GraphType
from territories_dashboard_lib.tracking_lib.logic import track_event

from .enums import MESHES_SHORT_TITLES
from .export import export_to_csv
from .format import format_data, format_indicator_value
from .models import UserFavoriteIndicator
from .payloads import (
    BasePayload,
    ComparisonQueryPayload,
    FlowsPayload,
    IndicatorTablePayload,
    OptionalComparisonQueryPayload,
    SubMeshOnlyPayload,
    SubMeshPayload,
)
from .query.commons import (
    get_mesh_column_name,
    get_sub_territories,
    get_territory_name,
    get_values_for_territory,
)
from .query.comparison import get_comparison_values_and_buckets
from .query.details import get_proportions_chart, get_values_for_submesh_territories
from .query.histogram import get_indicator_histogram_data
from .query.indicator_card import (
    get_geography_statistics_values_for_indicator,
    get_names_from_codes,
)
from .query.top_10 import get_indicator_top_10_data
from .query.utils import run_custom_query
from .table import (
    get_count_and_data_for_indicator_table,
    get_export_indicator_table_values,
)
from .utils import get_indicator_from_url_name


@get_indicator_from_url_name
def download_indicator_methodo_view(request, indicator):
    """View to download the methodology file for an indicator."""
    if not indicator.methodo_file:
        raise Http404("No methodology file available for this indicator.")

    response = HttpResponse(indicator.methodo_file, content_type="application/pdf")
    response["Content-Disposition"] = "attachment; filename=methodo.pdf"
    response = track_event(
        request=request,
        response=response,
        event_name=EventType.download,
        data={"indicator": indicator.name, "objet": "methodo", "type": "pdf"},
    )
    return response


@cache_control(max_age=3600)
@use_payload(SubMeshPayload)
@get_indicator_from_url_name
def indicator_statistics_view(request, indicator, payload):
    if payload.submesh == payload.territory.mesh:
        return HttpResponse(
            "les statiques ne sont pas disponibles si la maille est du même niveau que le territoire.",
            status=400,
        )
    filters = get_filters(request, indicator)
    query = get_geography_statistics_values_for_indicator(
        indicator,
        payload.territory,
        payload.submesh,
        filters,
    )
    rows = run_custom_query(query)
    if not rows:
        return HttpResponse("{}")
    data = rows[0]
    data = get_names_from_codes(data, payload.submesh)
    return HttpResponse(json.dumps(data))


def get_values(indicator, payload, filters):
    query = get_values_for_territory(
        indicator,
        payload.territory,
        filters,
    )
    rows = run_custom_query(query)
    results = {"values": rows}
    if payload.cmp_territory:
        query = get_values_for_territory(
            indicator,
            payload.cmp_territory,
            filters,
        )
        cmp_rows = run_custom_query(query)
        results["cmp_values"] = cmp_rows
    return results


def get_filters(request, indicator):
    filters = {}
    for dimension in indicator.dimensions.all():
        input_filters = request.GET.getlist(dimension.db_name)
        possible_filters = [f.db_name for f in dimension.filters.all()]
        filters[dimension.db_name] = [f for f in input_filters if f in possible_filters]
    return filters


@require_GET
@cache_control(max_age=3600)
@use_payload(OptionalComparisonQueryPayload)
@get_indicator_from_url_name
def indicator_values_view(request, indicator, payload):
    filters = get_filters(request, indicator)
    results = get_values(indicator, payload, filters)
    return HttpResponse(json.dumps(results))


@require_GET
@use_payload(OptionalComparisonQueryPayload)
@get_indicator_from_url_name
def indicator_values_export_view(request, indicator, payload):
    filters = get_filters(request, indicator)
    results = get_values(indicator, payload, filters)
    export_values = {}
    territory_name = get_territory_name(payload.territory)
    for value in results["values"]:
        export_values[value["annee"]] = {
            "Année": value["annee"],
            f"Valeur {territory_name} ({indicator.unite})": value["valeur"],
        }
        if indicator.unite_alternative and "valeur_alternative" in value:
            export_values[value["annee"]][
                f"Valeur {territory_name} ({indicator.unite_alternative})"
            ] = value["valeur_alternative"]
    tracking_objet = "historique"
    if results.get("cmp_values") is not None:
        cmp_territory_name = get_territory_name(payload.cmp_territory)
        for value in results["cmp_values"]:
            export_values[value["annee"]][
                f"Valeur {cmp_territory_name} ({indicator.unite})"
            ] = value["valeur"]
        tracking_objet = "comparaison-" + tracking_objet
    return export_to_csv(
        request, indicator, tracking_objet, list(export_values.values())
    )


@require_GET
@cache_control(max_age=3600)
@use_payload(SubMeshPayload)
@get_indicator_from_url_name
def indicator_submesh_territories_view(request, indicator, payload):
    filters = get_filters(request, indicator)
    data = get_values_for_submesh_territories(
        indicator, payload.submesh, payload.territory, filters
    )
    return HttpResponse(json.dumps(data))


@cache_control(max_age=3600)
@use_payload(BasePayload)
@get_indicator_from_url_name
def proportions_chart_view(request, indicator, payload):
    if indicator.dimensions.count() == 0:
        return HttpResponse(status=400)
    filters = get_filters(request, indicator)
    rows = get_proportions_chart(
        indicator,
        payload.territory,
        filters,
    )
    return HttpResponse(json.dumps({"values": rows}))


@require_GET
@use_payload(BasePayload)
@get_indicator_from_url_name
def indicator_proportions_export_view(request, indicator, payload):
    if indicator.dimensions.count() == 0:
        return HttpResponse(status=400)
    filters = get_filters(request, indicator)
    rows = get_proportions_chart(
        indicator,
        payload.territory,
        filters,
    )
    rows = [
        {"Dimension": r["label"], f"Valeur {indicator.unite}": r["data"][0]}
        for r in rows
    ]
    return export_to_csv(request, indicator, GraphType.repartition_dimension, rows)


@cache_control(max_age=3600)
@use_payload(SubMeshOnlyPayload)
@get_indicator_from_url_name
def indicator_histogram_view(request, indicator, payload):
    filters = get_filters(request, indicator)
    data = get_indicator_histogram_data(
        indicator,
        payload.territory,
        payload.submesh,
        filters,
    )
    return HttpResponse(json.dumps(data))


@cache_control(max_age=3600)
@use_payload(SubMeshOnlyPayload)
@get_indicator_from_url_name
def indicator_histogram_export_view(request, indicator, payload):
    filters = get_filters(request, indicator)
    data = get_indicator_histogram_data(
        indicator,
        payload.territory,
        payload.submesh,
        filters,
    )
    rows = []
    for index, decile in enumerate(data["deciles"]):
        row = {}
        next_decile = (
            format_indicator_value(data["deciles"][index + 1])
            if index < len(data["deciles"]) - 1
            else "+"
        )
        row["Décile"] = f"{format_indicator_value(decile)} - {next_decile}"
        row["Nombre de territoires"] = data["datasetsHistogramBarChart"]["data"][index][
            "y"
        ]
        row["Commentaire"] = data["datasetsHistogramBarChart"]["comments"][
            index
        ].replace("\n", " | ")
        rows.append(row)
    return export_to_csv(request, indicator, GraphType.repartition_valeurs, rows)


@cache_control(max_age=3600)
@use_payload(SubMeshOnlyPayload)
@get_indicator_from_url_name
def indicator_top_10_view(request, indicator, payload):
    filters = get_filters(request, indicator)
    data, _ = get_indicator_top_10_data(
        indicator,
        payload.territory,
        payload.submesh,
        filters,
    )
    return HttpResponse(json.dumps(data))


@require_GET
@use_payload(SubMeshOnlyPayload)
@get_indicator_from_url_name
def indicator_top_10_export_view(request, indicator, payload):
    filters = get_filters(request, indicator)
    _, csv_data = get_indicator_top_10_data(
        indicator,
        payload.territory,
        payload.submesh,
        filters,
    )
    return export_to_csv(request, indicator, GraphType.top_10, csv_data)


@require_GET
@cache_control(max_age=3600)
@use_payload(FlowsPayload)
def flows_view(request, payload):
    flows_table = f"{payload.prefix}_{payload.submesh}"

    # Query to get the latest year
    last_year_query = f"SELECT DISTINCT annee FROM {flows_table} ORDER BY annee DESC"
    last_year = run_custom_query(last_year_query)[0]["annee"]

    territories = get_sub_territories(
        submesh=payload.submesh, territory=payload.territory, with_center=True
    )
    territories_ids = ", ".join(f"'{t['code'].strip()}'" for t in territories)

    dimension_value = (
        f"{payload.dimension} as dimension"
        if payload.dimension
        else "'all' as dimension"
    )

    mesh_col = get_mesh_column_name(payload.submesh)
    # Values query
    values_query = f"""
        SELECT
            {mesh_col}_1 as territory_1_id,
            {mesh_col}_2 as territory_2_id,
            CAST(valeur AS int) as value,
            {dimension_value}
        FROM {flows_table} flows
        WHERE annee = {last_year}
        AND (
            {mesh_col}_1 IN ({territories_ids})
            OR {mesh_col}_2 IN ({territories_ids})
        )
        AND {mesh_col}_1 IS NOT NULL
        AND {mesh_col}_2 IS NOT NULL
    """
    row_values = run_custom_query(values_query)
    territories_dict = {
        t["code"]: {
            "name": t["name"],
            "code": t["code"],
            "center": json.loads(t["center"]),
        }
        for t in territories
    }

    external_territories_ids = set()
    for row in row_values:
        if row["territory_1_id"] not in territories_dict:
            external_territories_ids.add(row["territory_1_id"])
        if row["territory_2_id"] not in territories_dict:
            external_territories_ids.add(row["territory_2_id"])

    external_territories = []
    if external_territories_ids:
        external_territories = get_sub_territories(
            submesh=payload.submesh, codes=external_territories_ids, with_center=True
        )
    for t in external_territories:
        territories_dict[t["code"]] = {
            "name": f"{t['name']} (externe)",
            "code": t["code"],
            "center": json.loads(t["center"]),
        }

    return JsonResponse(
        {"flows": row_values, "territories": territories_dict}, status=200
    )


@require_GET
@cache_control(max_age=3600)
@use_payload(ComparisonQueryPayload)
@get_indicator_from_url_name
def comparison_histogram_view(request, indicator, payload):
    filters = get_filters(request, indicator)
    territories = get_sub_territories(
        submesh=payload.submesh, territory=payload.territory
    )
    cmp_territories = get_sub_territories(
        submesh=payload.submesh, territory=payload.cmp_territory
    )
    values, cmp_values, buckets = get_comparison_values_and_buckets(
        indicator, payload.submesh, territories, cmp_territories, filters
    )
    return JsonResponse(
        {
            "values": values,
            "comparedValues": cmp_values,
            "buckets": buckets,
        }
    )


@require_GET
@use_payload(ComparisonQueryPayload)
@get_indicator_from_url_name
def comparison_histogram_export_view(request, indicator, payload):
    filters = get_filters(request, indicator)
    territories = get_sub_territories(
        submesh=payload.submesh, territory=payload.territory
    )
    cmp_territories = get_sub_territories(
        submesh=payload.submesh, territory=payload.cmp_territory
    )
    values, cmp_values, buckets = get_comparison_values_and_buckets(
        indicator, payload.submesh, territories, cmp_territories, filters
    )
    rows = []
    territory_name = get_territory_name(payload.territory)
    cmp_territory_name = get_territory_name(payload.cmp_territory)
    for index, bucket in enumerate(buckets):
        row = {}
        row["Décile"] = (
            f"{format_indicator_value(bucket[0])} - {format_indicator_value(bucket[1])}"
        )
        row[f"{territory_name} - Nombre de {MESHES_SHORT_TITLES[payload.submesh]}s"] = (
            len(values[index + 1])
        )
        row[
            f"{cmp_territory_name} - Nombre de {MESHES_SHORT_TITLES[payload.submesh]}s"
        ] = len(cmp_values[index + 1])
        row[f"{territory_name} - échantillon de dix territoires"] = " | ".join(
            values[index + 1][:10]
        )
        row[f"{cmp_territory_name} - échantillon de dix territoires"] = " | ".join(
            cmp_values[index + 1][:10]
        )
        rows.append(row)
    return export_to_csv(request, indicator, GraphType.comparison_histogram, rows)


def get_label(props, indicator, key):
    labels = {
        "annee": "Année",
        "name": "Lieu",
        "territory_1": "Origine",
        "territory_2": "Destination",
        "valeur": "flux" if props.flows else indicator.unite,
        "valeur_alternative": indicator.unite_alternative,
        "dimension": "Dimension",
    }
    for dimension in indicator.dimensions.all():
        labels[dimension.db_name] = dimension.title
    return labels[key]


def get_pages(count, limit, current):
    length = count // limit
    if count % limit != 0 or count == 0:
        length += 1
    page_range = list(range(1, length + 1))
    pages = {
        "first": page_range[0] if page_range[0] != current else None,
        "last": page_range[-1] if page_range[-1] != current else None,
        "current": current,
        "before": current - 1 if current - 1 in page_range else None,
        "after": current + 1 if current + 1 in page_range else None,
        "only_one_page": page_range == [1],
    }
    return pages


def get_line_focus(payload: IndicatorTablePayload):
    if not payload.focus:
        return None
    limit = payload.limit
    previous_limit = payload.previous_limit
    if limit and previous_limit:
        if limit <= previous_limit:
            return 1
        else:
            return previous_limit + 1
    return None


@require_GET
@use_payload(IndicatorTablePayload)
@get_indicator_from_url_name
def indicator_details_table_view(request, indicator, payload):
    filters = get_filters(request, indicator)
    count, data = get_count_and_data_for_indicator_table(indicator, payload, filters)
    formated_data = [format_data(element, precise=True) for element in data]
    keys = [
        {"db": key, "label": get_label(payload, indicator, key)}
        for key in (formated_data[0].keys() if formated_data else [])
    ]
    last_year = None
    if data and "annee" in data[0].keys():
        last_year = max([d["annee"] for d in data])
    context = {
        "rows": formated_data,
        "keys": keys,
        "result_count": count,
        "pages": get_pages(count, payload.limit, payload.pagination),
        "props": payload,
        "last_year": last_year,
        "line_focus": get_line_focus(payload),
    }
    return render(
        request,
        "territories_dashboard_lib/website/pages/indicators/details/components/table.html",
        context,
    )


@require_GET
@use_payload(IndicatorTablePayload)
@get_indicator_from_url_name
def indicator_details_table_export_view(request, indicator, payload):
    filters = get_filters(request, indicator)
    table_values = get_export_indicator_table_values(indicator, payload, filters)
    return export_to_csv(request, indicator, GraphType.table, table_values)


@require_POST
@login_required
@get_indicator_from_url_name
def favorite_indicator_view(request, indicator):
    old_count = request.user.favorite_indicators.count()
    if indicator.favorited_by.filter(pk=request.user.pk).exists():
        indicator.favorited_by.remove(request.user)
        is_favorite = False
    else:
        order = request.user.favorite_indicators.count()
        UserFavoriteIndicator.objects.create(
            user=request.user, indicator=indicator, order=order
        )
        is_favorite = True
    new_count = request.user.favorite_indicators.count()
    update_left_menu = (new_count == 1 and old_count == 0) or new_count == 0

    html = loader.render_to_string(
        "territories_dashboard_lib/website/pages/indicators/components/favorite.html",
        {"is_favorite": is_favorite},
        request,
    )

    return JsonResponse(
        {"html": html, "updateLeftMenu": update_left_menu, "isFavorite": is_favorite}
    )


@require_POST
@login_required
def reorder_favorite_view(request, name):
    data = json.loads(request.body)
    indicator_id = data["indicator_id"]
    direction = data["direction"]

    try:
        current = UserFavoriteIndicator.objects.get(
            user=request.user, indicator_id=indicator_id
        )
    except UserFavoriteIndicator.DoesNotExist:
        return JsonResponse({"error": "not found"}, status=404)

    if direction == "up":
        neighbor = (
            UserFavoriteIndicator.objects.filter(
                user=request.user, order__lt=current.order
            )
            .order_by("-order")
            .first()
        )
    else:
        neighbor = (
            UserFavoriteIndicator.objects.filter(
                user=request.user, order__gt=current.order
            )
            .order_by("order")
            .first()
        )

    if neighbor:
        current.order, neighbor.order = neighbor.order, current.order
        current.save(update_fields=["order"])
        neighbor.save(update_fields=["order"])

    return HttpResponse()



from territories_dashboard_lib.indicators_lib.enums import MeshLevel
from territories_dashboard_lib.indicators_lib.query.utils import run_custom_query

from .models import Dashboard


def get_territory_meshes(territory_id: str, territory_mesh: MeshLevel):
    current_reg = None
    current_dep = None
    current_epci = None
    current_com = None
    if territory_mesh != MeshLevel.fr:
        query = f"""
        SELECT
            name_reg,
            name_dep,
            name_epci,
            name_com
        FROM arborescence_geo
        WHERE code_{territory_mesh} = '{territory_id}'
        LIMIT 1
        """
        row = run_custom_query(query)[0]
        if territory_mesh in [
            MeshLevel.reg,
            MeshLevel.dep,
            MeshLevel.epci,
            MeshLevel.com,
        ]:
            current_reg = row["name_reg"]
        if territory_mesh in [
            MeshLevel.dep,
            MeshLevel.epci,
            MeshLevel.com,
        ]:
            current_dep = row["name_dep"]
        if territory_mesh in [
            MeshLevel.epci,
            MeshLevel.com,
        ]:
            current_epci = row["name_epci"]
        if territory_mesh in [
            MeshLevel.com,
        ]:
            current_com = row["name_com"]
    territory_meshes = {
        MeshLevel.reg: current_reg,
        MeshLevel.dep: current_dep,
        MeshLevel.epci: current_epci,
        MeshLevel.com: current_com,
    }
    return territory_meshes


def make_filter(dashboard: Dashboard, territory_id: str, territory_mesh: MeshLevel):
    if dashboard.filters.count() == 0:
        return None
    territory_meshes = get_territory_meshes(territory_id, territory_mesh)
    filters = []
    for f in dashboard.filters.all():
        value = territory_meshes.get(f.mesh)
        if value:
            value = value.replace("'", "!'")  # RISON escape
            filter_string = f"""NATIVE_FILTER-{f.superset_id}:(__cache:(label:'{value}',validateStatus:!f,value:!('{value}')),extraFormData:(filters:!((col:{f.superset_col},op:IN,val:!('{value}')))),filterState:(label:'{value}',validateStatus:!f,value:!('{value}')),id:NATIVE_FILTER-{f.superset_id},ownState:())"""
            filters.append(filter_string)
    if not filters:
        return None
    return f"""({",".join(filters)})"""



from functools import wraps

from django.http import Http404
from django.shortcuts import get_object_or_404

from .models import Indicator


def get_indicator_from_url_name(view_func):
    """
    Decorator that retrieves an Indicator from the URL 'name' parameter.

    Replaces the 'name' parameter with the actual 'indicator' object.
    Raises Http404 if the indicator is not found or is not active.
    """

    @wraps(view_func)
    def wrapper(request, name, *args, **kwargs):
        indicator = get_object_or_404(Indicator, name=name)
        if not indicator.is_active:
            raise Http404("Indicator is not active.")
        return view_func(request, indicator=indicator, *args, **kwargs)

    return wrapper



# Generated by Django 5.2.11 on 2026-02-04 18:23

import martor.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('website_lib', '0005_mainconf_meshes'),
    ]

    operations = [
        migrations.AlterField(
            model_name='landingpage',
            name='body',
            field=martor.models.MartorField(help_text="Contenu de la page d'accueil au format Markdown pour la page d'accueil par défaut.", verbose_name='Contenu'),
        ),
        migrations.AlterField(
            model_name='landingpage',
            name='button_link',
            field=models.TextField(help_text="Lien du bouton d'action de la page d'accueil (pour la page d'accueil par défaut), au format markdown, par exemple : [découvrir les indicateurs](/indicateurs/)", verbose_name='Lien du bouton'),
        ),
        migrations.AlterField(
            model_name='mainconf',
            name='contact_email',
            field=models.TextField(blank=True, default=None, help_text="Uniquement si l'affichage de la bannière d'email de contact et d'inscription à la newsletter est activée.", null=True, verbose_name='Email de contact'),
        ),
        migrations.AlterField(
            model_name='mainconf',
            name='footer_navigation',
            field=models.TextField(default='', help_text='Navigation du footer (bas de page).\nMettre les liens au format markdown, un par ligne.\n\nSi le lien est interne, commencer et terminer le lien par un slash comme "/accueil/"\n\nPar exemple :\n\n[Plan du site](/plan-site/)\n[Accessibilité](/accessibilite/)\n[Mentions légales](/mentions-legales/)', verbose_name='Navigation du footer'),
        ),
        migrations.AlterField(
            model_name='mainconf',
            name='header_navigation',
            field=models.TextField(default='', help_text='Navigation du header (liste des onglets et sous-onglets en haut de page).\nMettre les liens au format markdown avec un slash à la fin du lien, mettre une ligne vide entre chaque rubrique.\n        \nUne rubrique peut contenir un seul lien, ou bien plusieurs, dans ce cas la rubrique sera sous forme de dropdown et il faut mettre un titre en premier. \n\nPar exemple : \n\n[Accueil](/accueil/)\n\n[Indicateurs territoriaux](/indicateurs/)\n\nÀ propos\n[Présentation](/presentation/)\n[Journal des versions](/journal/)', verbose_name='Navigation du header'),
        ),
        migrations.AlterField(
            model_name='mainconf',
            name='newsletter_link',
            field=models.TextField(blank=True, default=None, help_text="Uniquement si l'affichage de la bannière d'email de contact et d'inscription à la newsletter est activée.", null=True, verbose_name="Lien d'inscription à la newsletter"),
        ),
        migrations.AlterField(
            model_name='noticebanner',
            name='description',
            field=models.CharField(help_text='Texte à afficher dans la bannière à la suite du Titre.', max_length=255, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='noticebanner',
            name='link_href',
            field=models.CharField(blank=True, help_text="Optionnel. Le lien doit être un lien interne et commencer et terminer par '/', par exemple '/journal/'", max_length=255, null=True, verbose_name='Lien'),
        ),
        migrations.AlterField(
            model_name='noticebanner',
            name='link_text',
            field=models.CharField(blank=True, help_text='Optionnel. A remplir seulement si un lien est spécifié', max_length=64, null=True, verbose_name='Intitulé du lien'),
        ),
        migrations.AlterField(
            model_name='noticebanner',
            name='title',
            field=models.CharField(help_text='Sera affiché en gras au début de la bannière.', max_length=64, verbose_name='Titre'),
        ),
    ]



# Generated by Django 5.2.3 on 2025-06-18 13:03

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('cookie', models.TextField(help_text="Un cookie sans durée d'expiration est placé dans chaque nouvelle session, il possède une valeur aléatoire qui permet de tracer le navigateur du visiteur entre plusieurs visites.")),
                ('name', models.TextField(choices=[('download', 'Download')])),
                ('data', models.JSONField(blank=True, null=True)),
                ('url', models.TextField(help_text='URL brut de la page')),
                ('view', models.TextField(help_text='Nom de la vue dans Django')),
            ],
        ),
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('cookie', models.TextField(help_text="Un cookie sans durée d'expiration est placé dans chaque nouvelle session, il possède une valeur aléatoire qui permet de tracer le navigateur du visiteur entre plusieurs visites.")),
                ('territory_id', models.TextField(blank=True, help_text='ID du territoire principal sélectionné.', null=True)),
                ('territory_mesh', models.TextField(blank=True, choices=[('fr', 'National'), ('reg', 'Region'), ('dep', 'Department'), ('epci', 'Epci'), ('com', 'Town')], help_text='Maille du territoire principal sélectionné.', null=True)),
                ('submesh', models.TextField(blank=True, choices=[('fr', 'National'), ('reg', 'Region'), ('dep', 'Department'), ('epci', 'Epci'), ('com', 'Town')], help_text="Maille d'analyse sélectionnée.", null=True)),
                ('cmp_territory_id', models.TextField(blank=True, help_text="ID du territoire de comparaison, null s'il ne s'agit pas de la page comparaison.", null=True)),
                ('cmp_territory_mesh', models.TextField(blank=True, choices=[('fr', 'National'), ('reg', 'Region'), ('dep', 'Department'), ('epci', 'Epci'), ('com', 'Town')], help_text="Maille du territoire de comparaison, null s'il ne s'agit pas de la page comparaison.", null=True)),
                ('indicator', models.IntegerField(blank=True, help_text="ID de l'indicateur de la page détails, null s'il ne s'agit pas de la page détails, utile pour faire des jointures.", null=True)),
                ('indicator_name', models.TextField(blank=True, help_text="Nom de l'indicateur de la page détails, null s'il ne s'agit pas de la page détails, utile pour une analyse rapide.", null=True)),
                ('theme', models.IntegerField(blank=True, help_text='ID du thème de la page, utile pour faire des jointures.', null=True)),
                ('theme_name', models.TextField(blank=True, help_text='Nom du thème de la page, utile pour une analyse rapide.', null=True)),
                ('url', models.TextField(help_text='URL brut de la page')),
                ('view', models.TextField(help_text='Nom de la vue dans Django')),
            ],
        ),
    ]



class AdminInfoMixin:
    admin_info = ""

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context["subtitle"] = self.admin_info
        return super().changelist_view(request, extra_context=extra_context)



from django.db import models

from territories_dashboard_lib.commons.models import CommonModel
from territories_dashboard_lib.indicators_lib.enums import MeshLevel


class Dashboard(CommonModel):
    superset_id = models.TextField(
        unique=True,
        verbose_name="Embed ID de Superset",
        help_text="Un utilisateur administrateur de l'instance Superset doit accéder aux paramètres du dashboard et cliquer sur 'embed dashboard' pour récupérer cet ID nécessaire à la connexion avec l'instance Superset.",
    )
    short_name = models.TextField(
        unique=True,
        verbose_name="Nom court",
        help_text="Pour l'URL, ne mettre que des lettres minuscules sans accents et des tirets",
    )
    label = models.TextField(
        unique=True,
        verbose_name="Label",
        help_text="Nom du dashboard à afficher dans la liste.",
    )
    order = models.IntegerField(
        default=0,
        verbose_name="Numéro d'ordre",
        help_text="Numéro d'ordre dans la dropdown de sélection, les dashboards sont triés du plus petit numéro au plus grand. L'ordre démarre à 0.",
    )

    def __str__(self):
        return self.label


class Filter(CommonModel):
    dashboard = models.ForeignKey(
        Dashboard, on_delete=models.CASCADE, related_name="filters"
    )
    superset_id = models.TextField(
        null=True,
        blank=True,
        help_text="ID du native filter du territoire, qui permettra d'initialiser les filtres du dasbhoard selon le territoire sélectionné dans l'application. Pour le récupérer c'est un petit parcours du combattant : aller sur le dashboard dans superset, cliquer sur modifier le dashboard, cliquer sur les trois petits points, puis sur modifier les propriétés. Cliquer sur avancé, copier le json et le coller dans un site comme : https://jsonformatter.curiousconcept.com/ pour mieux le voir. Chercher 'global_chart_configuration' puis 'native_filter_configuration'. Chercher le native filter lié au choix du territoire. Copier l'id qui dans son nom, le nom est de la forme NATIVE_FILTER-ID. ",
    )
    superset_col = models.TextField(
        null=True,
        blank=True,
        help_text="Nom de la colonne en base de données liée au filtre sur le territoire et utilisé dans Superset. Pour trouver le nom, on peut faire les mêmes étapes que pour le 'superset_id' et chercher 'column' dans les paramètres json du native filter.",
    )
    mesh = models.TextField(
        choices=MeshLevel.choices,
        null=True,
        blank=True,
        help_text="Maille pour laquelle s'effectue le filtre sur le territoire.",
    )



# Generated by Django 5.2.3 on 2025-06-19 07:13

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('indicators_lib', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='GeoFeature',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.TextField(help_text='attention le nom doit être exactement le même que celui en base.', verbose_name='Nom de la table en base')),
                ('title', models.TextField(default='à compléter', verbose_name='Titre à afficher')),
                ('unite', models.TextField(verbose_name='Unité au pluriel')),
                ('geo_type', models.TextField(choices=[('point', 'Point'), ('line', 'Line'), ('polygon', 'Polygon')], default='point', verbose_name='Type de géométrie')),
                ('point_icon_svg', models.TextField(blank=True, help_text="Icone SVG a utilisé pour les points.<br/>Laisser vide pour les autres types de géométrie.<br/>Veuillez charger un fichier provenant des <a href='/'>Material Icons</a> de Google.", null=True, verbose_name='Icone des points')),
                ('color', models.TextField(default='#000000', help_text="Couleur des données sur la date, à renseigner au format hexadécimal, par exemple: '#05B2F9'", verbose_name='Couleur')),
                ('show_on_fr_level', models.BooleanField(default=True, help_text='Décochez si la quantité de données est trop importante pour le niveau France entière.', verbose_name='Afficher au niveau France entière')),
                ('indicator', models.ForeignKey(help_text="Cette donnée géographique sera ajoutée à la carte de l'indicateur.", on_delete=django.db.models.deletion.CASCADE, related_name='geo_features', to='indicators_lib.indicator')),
            ],
            options={
                'verbose_name': 'Données de la carte',
                'ordering': ('indicator', 'name'),
            },
        ),
        migrations.CreateModel(
            name='GeoElement',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.TextField(verbose_name='Nom de la colonne en DB')),
                ('label', models.TextField()),
                ('filterable', models.BooleanField(default=True, verbose_name='Filtrable')),
                ('geo_feature', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='items', to='geo_lib.geofeature')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import base64
import datetime
import gzip
import json

from django.http import HttpResponse, JsonResponse
from django.shortcuts import get_object_or_404
from django.views.decorators.cache import cache_control
from django.views.decorators.http import require_GET
from psycopg2.sql import SQL

from territories_dashboard_lib.commons.decorators import use_payload
from territories_dashboard_lib.geo_lib.payloads import (
    GeoFeaturesPayload,
    MainTerritoryParams,
    SearchTerritoriesParams,
    TerritoriesParams,
    TerritoryFeaturePayload,
)
from territories_dashboard_lib.indicators_lib.enums import (
    FRANCE_GEOLEVEL_TITLES,
    MeshLevel,
)
from territories_dashboard_lib.indicators_lib.query.commons import (
    get_sub_territories,
)
from territories_dashboard_lib.indicators_lib.query.utils import (
    format_sql_codes,
    run_custom_query,
)
from territories_dashboard_lib.website_lib.conf import get_meshes_for_current_project

from .enums import GeoFeatureType
from .models import GeoFeature


class DateTimeEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, (datetime.date, datetime.datetime)):
            return (
                obj.date().isoformat()
                if isinstance(obj, datetime.datetime)
                else obj.isoformat()
            )
        return super().default(obj)


@require_GET
@cache_control(max_age=3600)
@use_payload(GeoFeaturesPayload)
def geo_features_view(request, payload):
    geo_feature = get_object_or_404(GeoFeature, id=payload.feature)
    last = payload.last
    limit = payload.limit

    last_year_query = (
        f"SELECT DISTINCT annee FROM {geo_feature.name} ORDER BY annee DESC"
    )
    years = run_custom_query(last_year_query)
    last_year = years[0].get("annee")

    territories = get_sub_territories(
        submesh=payload.submesh, territory=payload.territory
    )
    territories_ids = [t["code"] for t in territories]
    col_id_name = f"code_{payload.submesh}"
    columns = [el.name for el in geo_feature.items.all()]
    if geo_feature.color_column:
        columns.append(geo_feature.color_column)
    if geo_feature.size_column:
        columns.append(geo_feature.size_column)

    coma = ", " if columns else ""
    select_geo_code = (
        f", geo.{col_id_name} AS geo_code"
        if geo_feature.geo_type == GeoFeatureType.point
        else ""
    )

    if geo_feature.geo_type == GeoFeatureType.point:
        where_clause = f"WHERE geo.{col_id_name} IN ('{"', '".join(territories_ids)}')"
    else:
        where_clause = (
            f"JOIN contours_carto_{payload.territory.mesh}_pe contours ON ST_intersects(geo.geometry, contours.geometry)\n"
            f"WHERE contours.code IN {payload.territory.sql_codes}"
        )
    where_clause += f" AND annee = {last_year}"

    pagination = f"AND order_id > {last or 0} ORDER BY order_id ASC LIMIT {limit}"

    query = (
        f"SELECT ST_asgeojson(geo.geometry) AS geojson, order_id{coma + ', '.join(columns)}\n"
        f"{select_geo_code}\n"
        f"FROM {geo_feature.name} AS geo\n"
        f"{where_clause} {pagination}"
    )

    # Execute the query
    results = run_custom_query(query)

    items = geo_feature.items.all()
    data = []
    for r in results:
        properties = {el.label: r.get(el.name) for el in items}
        if geo_feature.color_column or geo_feature.size_column:
            properties["display"] = {}
        if geo_feature.color_column:
            properties["display"]["color"] = r.get(geo_feature.color_column)
        if geo_feature.size_column:
            properties["display"]["size"] = r.get(geo_feature.size_column)
        d = {
            "type": "Feature",
            "id": r.get("geo_code"),
            "properties": properties,
            "geometry": json.loads(r.get("geojson")),
        }
        data.append(d)

    last_queried_order_id = results[-1]["order_id"] if results else None

    compressed_data = gzip.compress(json.dumps(data, cls=DateTimeEncoder).encode())
    compressed_base64 = base64.b64encode(compressed_data).decode()

    return JsonResponse({"data": compressed_base64, "last": last_queried_order_id})


@require_GET
@cache_control(max_age=3600)
@use_payload(MainTerritoryParams)
def main_territory_view(request, payload):
    codes = ", ".join(f"'{code.strip()}'" for code in payload.geo_id)

    query = f"""
        SELECT code as id, ST_asgeojson(geometry) AS polygon
        FROM contours_carto_{payload.geo_level}_pe
        WHERE code IN ({codes})
        """

    results = run_custom_query(query)

    data = [
        {
            "id": r.get("id"),
            "polygon": json.loads(r.get("polygon")),
        }
        for r in results
    ]

    return JsonResponse(data, safe=False)


@require_GET
@cache_control(max_age=3600)
@use_payload(TerritoriesParams)
def precise_view(request, payload):
    mesh_level = payload.mesh
    territories_ids = payload.territories

    if not mesh_level:
        return JsonResponse({"error": "Missing 'mesh' parameter"}, status=400)

    query = f"""
        SELECT code AS id, ST_asgeojson(geometry) AS polygon
        FROM contours_carto_{mesh_level}
        WHERE code IN ('{"', '".join(territories_ids)}')
        """

    results = run_custom_query(query)

    territories = {r.get("id"): json.loads(r.get("polygon")) for r in results}

    return JsonResponse(territories)


@require_GET
@cache_control(max_age=3600)
@use_payload(TerritoryFeaturePayload)
def territories_view(request, payload):
    if payload.codes:
        territories_ids = payload.codes
    else:
        territories = get_sub_territories(
            submesh=payload.submesh, territory=payload.territory
        )
        territories_ids = [t["code"] for t in territories]
    query = f"""
        SELECT code AS id, ST_asgeojson(geometry) AS polygon
        FROM contours_carto_{payload.submesh}_pe
        WHERE code IN {format_sql_codes(territories_ids)}
        """

    results = run_custom_query(query)

    data = [
        {"type": "Feature", "id": r.get("id"), "geometry": json.loads(r.get("polygon"))}
        for r in results
    ]
    return JsonResponse(data, safe=False)


def _fill_territory_li(code, name, mesh):
    label = name if mesh == MeshLevel.fr else f"{name} - {code}"
    return f"""<li data-code="{code}" data-name="{name}" data-mesh="{mesh}" onclick="chooseTerritory(this)"><button aria-label="Choisir {label}">{label}</button></li>"""


@require_GET
@cache_control(max_age=3600)
@use_payload(SearchTerritoriesParams)
def search_territories_view(request, payload):
    mesh = payload.mesh
    search = payload.search
    offset = payload.offset
    if mesh == MeshLevel.fr:
        lis = []
        for code, name in FRANCE_GEOLEVEL_TITLES.items():
            li = _fill_territory_li(code, name, mesh)
            lis.append(li)
        return HttpResponse("\n".join(lis))
    table_name = f"arbo_{mesh}"
    pagination = 20
    include_ept = (
        mesh == MeshLevel.epci and MeshLevel.ept in get_meshes_for_current_project()
    )
    if include_ept:
        query = SQL(f"""
        SELECT code, name, mesh FROM (
            SELECT DISTINCT ON (code, name) code, name, mesh FROM (
                SELECT code, name, 'epci' as mesh FROM arbo_epci
                WHERE unaccent(name) || code ILIKE unaccent(%s)
                AND code !~ '^[A-Z]{{3}}$'
                UNION ALL
                SELECT code, name, 'ept' as mesh FROM arbo_ept
                WHERE unaccent(name) || code ILIKE unaccent(%s)
                AND code !~ '^[A-Z]{{3}}$'
            ) AS raw
            ORDER BY code, name, mesh
        ) AS combined
        ORDER BY name, code
        LIMIT {pagination} OFFSET {offset * pagination};
        """)
    else:
        query = SQL(f"""
        SELECT DISTINCT code, name, '{mesh}' as mesh FROM {table_name}
        WHERE unaccent(name) || code ILIKE unaccent(%s)
        AND code !~ '^[A-Z]{{3}}$'
        ORDER BY name, code
        LIMIT {pagination} OFFSET {offset * pagination};
        """)
    search_param = f"%{search}%"
    query_params = [search_param]
    if include_ept:
        query_params = [search_param, search_param]
    territories = run_custom_query(query, query_params)
    lis = []
    for territory in territories:
        li = _fill_territory_li(territory["code"], territory["name"], territory["mesh"])
        lis.append(li)
    if len(territories) == pagination:
        li = f"""<li data-type="load"><button onclick="loadMoreTerritories(this)" data-mesh="{mesh}" data-offset="{offset + 1}" class="fr-btn fr-btn--secondary fr-btn--sm">Charger plus de résultats</button></li>"""
        lis.append(li)
    return HttpResponse("\n".join(lis))



import re

from martor.utils import markdownify


def put_daily_motion_iframe(html_text):
    pattern = re.compile(
        r"<a href=\"(https://geo\.dailymotion\.com/player/x14squ\.html\?video=[a-z0-9]+)\">https://geo\.dailymotion\.com/player/x14squ\.html\?video=[a-z0-9]+</a>",
        re.MULTILINE,
    )

    def replacer(match):
        full_url = match.group(1)
        return f'<iframe src="{full_url}" height="270" width="480"></iframe>'

    return pattern.sub(replacer, html_text)


def markdown_content_to_html(content):
    return put_daily_motion_iframe(markdownify(content))



# Generated by Django 5.2.11 on 2026-02-10 16:00

import django.contrib.postgres.fields
import territories_dashboard_lib.website_lib.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('website_lib', '0006_alter_landingpage_body_alter_landingpage_button_link_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mainconf',
            name='footer_navigation',
            field=models.TextField(default='', help_text='Navigation du footer (bas de page).<br/><br/>\n\nMettre les liens au format markdown, un par ligne.<br/><br/>\n\nSi le lien est interne, commencer et terminer le lien par un slash comme "/accueil/"<br/><br/>\n\nPar exemple :<br/><br/>\n\n[Plan du site](/plan-site/)<br/>\n[Accessibilité](/accessibilite/)<br/>\n[Mentions légales](/mentions-legales/)', verbose_name='Navigation du footer'),
        ),
        migrations.AlterField(
            model_name='mainconf',
            name='header_navigation',
            field=models.TextField(default='', help_text='Navigation du header (liste des onglets et sous-onglets en haut de page).<br/><br/>\n\nMettre les liens au format markdown avec un slash à la fin du lien, mettre une ligne vide entre chaque rubrique.<br/><br/>\n        \nUne rubrique peut contenir un seul lien, ou bien plusieurs, dans ce cas la rubrique sera sous forme de dropdown et il faut mettre un titre en premier.<br/><br/>\n\nPar exemple :<br/><br/>\n\n[Accueil](/accueil/)<br/><br/>\n\n[Indicateurs territoriaux](/indicateurs/)<br/><br/>\n\nÀ propos<br/>\n[Présentation](/presentation/)<br/>\n[Journal des versions](/journal/).\n', verbose_name='Navigation du header'),
        ),
        migrations.AlterField(
            model_name='mainconf',
            name='meshes',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.TextField(choices=[('fr', 'Fr'), ('reg', 'Reg'), ('dep', 'Dep'), ('epci', 'Epci'), ('com', 'Com'), ('aom', 'Aom'), ('ept', 'Ept')]), default=territories_dashboard_lib.website_lib.models.get_default_meshes, help_text="Liste des mailles territoriales activées sur la plateforme, la liste des mailles disponible est : ['fr', 'reg', 'dep', 'epci', 'com', 'aom', 'ept'].<br/>L'ajout de nouvelles mailles demande une modification de la base de données des indicateurs ainsi que du code.", size=None, verbose_name='Mailles'),
        ),
    ]



from territories_dashboard_lib.indicators_lib.models import Theme


def get_favorite_theme():
    return Theme(name="favoris", title="Indicateurs favoris")


def get_favorite_subtheme(indicators):
    return {
        "name": "tableau-de-bord",
        "title": "Mon tableau de bord",
        "indicators": indicators,
    }



# Generated by Django 5.2.3 on 2025-06-26 12:58

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracking_lib', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='created_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name='page',
            name='created_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]



# Generated by Django 5.2.11 on 2026-02-04 18:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geo_lib', '0004_geofeature_legend'),
    ]

    operations = [
        migrations.AlterField(
            model_name='geoelement',
            name='filterable',
            field=models.BooleanField(default=True, help_text="Si coché, l'attribut est filtrable par l'utilisateur dans le volet de gestion des filtres.", verbose_name='Filtrable'),
        ),
        migrations.AlterField(
            model_name='geoelement',
            name='linked_to_indicator',
            field=models.BooleanField(default=False, help_text="Si coché, le filtre de l'indicateur avec le même nom sera relié au filtre de la carte.", verbose_name="Relié aux filtres de l'indicateur"),
        ),
        migrations.AlterField(
            model_name='geofeature',
            name='color',
            field=models.TextField(default='#000000', help_text="Couleur des données sur la carte, à renseigner au format hexadécimal, par exemple: '#05B2F9'", verbose_name='Couleur'),
        ),
        migrations.AlterField(
            model_name='geofeature',
            name='name',
            field=models.TextField(help_text='Attention, le nom doit être exactement le même que celui en base.', verbose_name='Nom de la table en base'),
        ),
        migrations.AlterField(
            model_name='geofeature',
            name='point_icon_svg',
            field=models.TextField(blank=True, help_text="Icone SVG à utiliser pour les points.<br/>Laisser vide pour les autres types de géométrie.<br/>Veuillez charger un fichier provenant des <a href='https://fonts.google.com/icons'>Material Icons</a> de Google.", null=True, verbose_name='Icone des points'),
        ),
        migrations.AlterField(
            model_name='geofeature',
            name='show_on_fr_level',
            field=models.BooleanField(default=True, help_text='Décochez si la quantité de données est trop importante pour le niveau France.', verbose_name='Afficher au niveau France'),
        ),
    ]
